import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.IEventAddedListener;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.events.DeadlineMissedEvent;
import yartiss.engine.simulation.events.JobExecutionEndEvent;
import yartiss.engine.simulation.io.many.ITasksetsReader;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.io.one.SimulationParametersIOFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;


public class Temp {

	/**
	 * @param args
	 * @throws Exception 
	 * @throws FileNotFoundException 
	 */
	public static void main(String[] args) throws FileNotFoundException, Exception {
		String file ="C:\\Users\\Younes\\Desktop\\yasmina";
		main1(file);

		String file2 = "C:\\Users\\Younes\\YounesDev\\YARTISS\\yartiss-engine\\src\\test\\resources\\hidden_systems\\100000_systems_pr_10";
		//main2(file2);
	}

	public static void main1(String file) throws FileNotFoundException, Exception {
		final SimulationParameters params = SimulationParametersIOFactory.newSimulationParametersReaderInstance("txt").read(file);    
		for (ITask task : params.getTaskSet()) {
			System.out.println("t"+task.getId()+" : "+responsetime(params.getTaskSet(), task, params.getEnergyProfile(), task.getFirstRelease()));
		}


		System.out.println("--------------------------");
		long simDuarion = 0;
		for (ITask task : params.getTaskSet()) {
			simDuarion = Math.max(simDuarion, task.getDeadline()+1);
		}

		final List<Event> events = new ArrayList<Event>();

		final Simulation sim = new Simulation(params);
		sim.addEventAddedListerner(new IEventAddedListener() {
			@Override public void perfomredAction(Event e) {
				events.add(e);
			}
		});
		sim.simulate();

		for (ITask task : params.getTaskSet()) {
			long rt = responsetime(params.getTaskSet(), task, params.getEnergyProfile(), task.getFirstRelease());
			boolean found = false;
			boolean deadline = false;
			for (Event event : events) {
				if(event instanceof DeadlineMissedEvent)	deadline = true;
				if(rt==event.getDate() && event.getTask()==task  && event instanceof JobExecutionEndEvent){
					found = true;
					break;
				}
			}

			if(!found && rt <= task.getDeadline() && !deadline){
				System.out.println(task);
				for (ITask taskk : params.getTaskSet()) {
					System.out.println(responsetime(params.getTaskSet(), taskk, params.getEnergyProfile(), taskk.getFirstRelease())+"    "+taskk);
				}
				System.out.println("--------- setId -------------------------");
				System.exit(0);
			}
		}

	}

	public static void main2(String systemsSource) throws Exception{

		ITask[] tasks = null;
		long simDuarion = 0;
		File file = new File(systemsSource);
		ITasksetsReader reader = TasksetsIOFactory.newTasksetsReaderInstance("txt",file);
		int i=0;

		System.out.println("start");
		while((tasks = reader.nextTaskset()) != null){

			i++;
			final ISchedulingPolicy policy = SchedulingPolicyFactory.getInstance("EH_PFP_1");
			final IEnergyProfile ep = EnergyProfileFactory.getInstance("default", 10, Integer.MAX_VALUE, 0, 0);
			final ITaskSet set = policy.createTaskSet();
			final int setId = i;

			simDuarion = 0;
			for (ITask task : tasks) {
				set.addTask(task);
				simDuarion = Math.max(simDuarion, task.getDeadline()+1);
			}

			System.out.println(i+" : "+simDuarion);

			final List<Event> events = new ArrayList<Event>();

			final SimulationParameters params = new SimulationParameters(policy, (int)simDuarion, ep, set, 1, true, true);
			final Simulation sim = new Simulation(params);
			sim.addEventAddedListerner(new IEventAddedListener() {
				@Override public void perfomredAction(Event e) {
					events.add(e);
				}
			});

			if(i==20000) 
				SimulationParametersIOFactory.newSimulationParametersWriterInstance("txt").write(params,  "C:\\Users\\Younes\\Desktop\\file");

			sim.simulate();
			for (ITask task : params.getTaskSet()) {
				long rt = responsetime(params.getTaskSet(), task, params.getEnergyProfile(), task.getFirstRelease());
				boolean found = false;
				boolean deadline = false;

				for (Event event : events) {
					if(event instanceof DeadlineMissedEvent)	deadline = true;
					if(rt==event.getDate() && event.getTask()==task  && event instanceof JobExecutionEndEvent){
						found = true;
						break;
					}
				}

				if(!found && rt <= task.getDeadline() && !deadline ){
					System.out.println(task);
					for (ITask taskk : params.getTaskSet()) {
						System.out.println(responsetime(params.getTaskSet(), taskk, params.getEnergyProfile(), taskk.getFirstRelease())+"    "+taskk);
					}
					System.out.println("--------- "+setId+" -------------------------");
					SimulationParametersIOFactory.newSimulationParametersWriterInstance("txt").write(params,  "C:\\Users\\Younes\\Desktop\\file");
					System.exit(0);
				}
			}

		}
		reader.close();
		System.out.println("finish");
	}

	public static long responsetime(ITaskSet tasks, ITask task_i, IEnergyProfile battery, long t){
		double w_n;
		double w_n1 = t;
		double Ei, Ci, Pr, Ej, Cj, Tj, E0, Eii,Ejj,xj;

		Pr = battery.getPowerRate();
		E0 = battery.getStartEnergyLevel();
		Ei = task_i.getWcee();
		Ci = task_i.getWcet();
		Eii= Math.max(Ei, Ci * Pr);

		do {
			w_n = w_n1;
			double energyDemand = Eii - E0;
			for (ITask task_j : tasks.getHeadSet(task_i, false)) {
				Ej = task_j.getWcee();
				Cj = task_j.getWcet();
				Tj = task_j.getPeriod();
				Ejj= Math.max(Ej, Cj * Pr);
				xj = task_j.getNextActivation(t-1);

				energyDemand += Math.ceil(  (t + w_n - xj)/Tj ) * Ejj;
			}
			w_n1 = Math.ceil(energyDemand/battery.getPowerRate());
			System.out.println("w_n1="+Math.round(w_n1));
		}while(w_n1 != w_n);
		System.out.println("######################################");
		return Math.round(w_n1); 
	}



}
