/*

    Real-Time Systems Simulator v2 si an event based real-time systems
simulator
    Copyright (C) <2010>  Damien Masson -- visit <http://dajam.fr/>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>

 */
package yartiss.engine.simulation;

import yartiss.engine.simulation.system.ITaskSet;

/**
 * This class models the parameters for a simulation
 * 
 * @author Damien Masson
 * 
 */
public class SimulationParameters {

	private final ITaskSet tasks;
	private final ISchedulingPolicy policy;
	private final long simulationEnd;
	private final IEnergyProfile energyProfile;
	private final int nbProc;
	private final boolean isEnergyAware;
	private final boolean isHardRealtime;

	/**
	 * 
	 * @return the tasks set to simulate
	 */
	public ITaskSet getTaskSet() {
		return tasks;
	}

	/**
	 * 
	 * @return the scheduling policy for this simulation
	 */
	public ISchedulingPolicy getSchedulingPolicy() {
		return policy;
	}

	/**
	 * 
	 * @return the duration of the simulation in time unit
	 */
	public long getSimulationEnd() {
		return simulationEnd;
	}
	
	/**
	 * @return the energy profile
	 */
	public IEnergyProfile getEnergyProfile() {
		return energyProfile;
	}

	public SimulationParameters(ISchedulingPolicy policy, int simEnd, IEnergyProfile ep, ITaskSet taskSet, int nbProc, boolean isEnergyAware, boolean hard){
		this.nbProc = nbProc;
		this.policy = policy;
		this.simulationEnd = simEnd;
		this.energyProfile = ep;
		this.tasks = taskSet;
		this.isEnergyAware = isEnergyAware;
		this.isHardRealtime = hard;
	}

	/**
	 * @return the nbProc
	 */
	public int getNbProc() {
		return nbProc;
	}

	/**
	 * @return the isEnergyAware
	 */
	public boolean isEnergyAware() {
		return isEnergyAware;
	}

	/**
	 * @return the isHardRealtime
	 */
	public boolean isHardRealtime() {
		return isHardRealtime;
	}
}
