package yartiss.engine.simulation.comparison;

import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.SwingWorker;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.io.many.ITasksetsReader;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.framework.concurrency.Consumer;
import yartiss.framework.concurrency.IElementConsumedListener;
import yartiss.framework.concurrency.MultiThreadProducer;
import yartiss.framework.concurrency.QueueEntry;


public class ComparisonLauncher {

	private ComparisonConfiguration config;
	private Consumer<Simulation> consumer;
	private MultiThreadProducer<Simulation> producer;
	private final BoundedRangeModel globalProgressModel = new DefaultBoundedRangeModel(0, 0, 0, 100000000);

	public ComparisonLauncher(ComparisonConfiguration config){
		setConfiguration(config);
	}
	public void setConfiguration(ComparisonConfiguration config){
		this.config = config;
	}

	/**
	 * @return the globalProgressModel
	 */
	public BoundedRangeModel getGlobalProgressModel() {
		return globalProgressModel;
	}


	public void launch(){
		System.out.println("start command");

		getGlobalProgressModel().setValue(0);

		final long startDate = System.currentTimeMillis();
		new Thread(new Runnable() {
			int count;
			int total;
			boolean finish = false;
			public void run() {
				count = 0;
				finish = false;
				SwingWorker<Void, String> task = new SwingWorker<Void, String>() {
					private final BlockingQueue<QueueEntry<Simulation>> queue = new LinkedBlockingQueue<QueueEntry<Simulation>>(10000);
					@Override	protected Void doInBackground() throws Exception {

						producer = new MultiThreadProducer<Simulation>(queue);
						consumer = new Consumer<Simulation>(queue, new IElementConsumedListener<Simulation>() {
							@Override
							public void consume(Simulation sim) {  
								count ++;

								if(finish && count==total){
									producer.produce(null);
								}


								getGlobalProgressModel().setValue(getGlobalProgressModel().getValue()+1);
								//System.out.println((((double)globalProgressModel.getValue())/globalProgressModel.getMaximum())*100);
							}
						});
						consumer.addFinishListener(new IElementConsumedListener<Simulation>() {
							@Override	public void consume(Simulation element) {

								System.out.println("finish : count="+count);
								long endtDate = System.currentTimeMillis();
								System.out.println("total simulation duration : "+ (endtDate-startDate)/1000);
							}
						});

						consumer.start();

						for (final ComparisonScenario s : config.getComparisonScenarios()) {
							System.out.println("start ... producing");
							for (ISchedulingPolicy policy : config.getSchedulingPolicies()) {
								System.out.println("systems file ="+ config.getSystemsSource());
								ITasksetsReader reader = null;
								if(config.getSystemsSource().contains("file:") || config.getSystemsSource().contains("jar:")){	 
									URL url = new URL(config.getSystemsSource());
									reader = TasksetsIOFactory.newTasksetsReaderInstance("txt", url.openStream());
								}
								else {

									if(config.getSystemsSource().endsWith(".xml"))
										reader = TasksetsIOFactory.newTasksetsReaderInstance("xml",config.getSystemsSource());
									else
										reader = TasksetsIOFactory.newTasksetsReaderInstance("txt",config.getSystemsSource());
								}

								int nbEntries = 0;
								System.out.println("-----------" + policy.getPolicyName()+ " :  started");
								ITask[] tasks = null;
								while((tasks = reader.nextTaskset()) != null){

									nbEntries ++;
									final List<Integer> paramss = policy.getParameters()==null ? null : new LinkedList<Integer>(policy.getParameters());
									final ISchedulingPolicy policy2 = SchedulingPolicyFactory.getInstance(policy.getPolicyName());
									final ITask[] tasks2 = tasks;
									final int id = nbEntries;
									policy2.setParameters(paramss);
									final IEnergyProfile fep = EnergyProfileFactory.getInstance("defaultEnergyProfile", (int)s.getEnergyProfile().getPowerRate(), (int)s.getEnergyProfile().getMaxEnergyLevel(), (int)s.getEnergyProfile().getMinEnergyLevel(), (int)s.getEnergyProfile().getStartEnergyLevel());
									producer.produce(new Callable<Simulation>() {
										@Override
										public Simulation call() throws Exception {
											Simulation sim = new Simulation(buildSimulationParameters(fep, policy2, tasks2, id, s.getProcCount()));
											sim.simulate();
											return sim;
										}
									});
								}
								total = nbEntries*config.getSchedulingPolicies().size()*config.getComparisonScenarios().size();
								reader.close();
								getGlobalProgressModel().setMaximum(total);
							}
						}
						finish = true;
						System.out.println("finish producing : "+total);
						return null;
					}
				};
				task.execute();
			}
		}).start();

	}

	private SimulationParameters buildSimulationParameters(IEnergyProfile energyProfile, ISchedulingPolicy policy, ITask[] tasks, int id, int nbProc) {
		ITaskSet taskset = policy.createTaskSet();
		taskset.setId(id);
		for (ITask task : tasks) {
			taskset.addTask(task.cloneTask());
		}
		return new SimulationParameters(policy, (int)config.getSimulationDuration() , energyProfile, taskset,nbProc, true, true);
	}


}
