package yartiss.engine.simulation.io;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.io.one.SimulationParametersIOFactory;
import yartiss.engine.simulation.system.ITask;

public class SimulationWriter {
	
	private FileOutputStream logWriter;
	private Map<ITask, Integer> taksIds = new HashMap<ITask, Integer>();
	
	public void open(String file){
		try {
			if(logWriter != null)
				logWriter.close();
			taksIds.clear();
			logWriter = new FileOutputStream(file);
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void writeParameters(SimulationParameters parameters) {
		try {
			SimulationParametersIOFactory.newSimulationParametersWriterInstance("txt").write(parameters,  logWriter);
			int id = 1;
			for (ITask task : parameters.getTaskSet()) {
				taksIds.put(task, id);
				id++;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void writeEvent(Event event){
		try {
			logWriter.write((event.getDate() +"\t"+event.getMetric(Event.EnergyLevel)+"\tt"+ taksIds.get(event.getTask())+"\t"+event+"\n").getBytes());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void close(){
		try {
			if(logWriter != null)
				logWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
