package yartiss.engine.simulation.io.many;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.SystemMetrics;

public class TextTasksetsWriter implements ITasksetsWriter{

	
	private final Writer writer;
	long Pr;

	public TextTasksetsWriter(String dest, long pr) throws IOException {
		writer = new FileWriter(dest);
		Pr = pr;
	}

	public TextTasksetsWriter(OutputStream dest, long pr) {
		writer = new OutputStreamWriter(dest);
		Pr = pr;
	}

	public TextTasksetsWriter(File dest, long pr) throws IOException {
		writer = new FileWriter(dest);
		Pr = pr;
	}


	@Override
	public void write(ITask[] taskset) throws IOException {
		write(Arrays.asList(taskset));
	}
	
	
	@Override
	public void write(List<ITask> taskset) throws IOException {
		writer.write(Pr + " " + SystemMetrics.utilisation(taskset) + " "+ SystemMetrics.energyUtilization(taskset, Pr) + "\n");
		for (ITask task : taskset) {
			writer.write("firstrelease : " + task.getFirstRelease()
					+ " wcet : " + task.getWcet() + " wcee : " + task.getWcee()
					+ " period : " + task.getPeriod() + " deadline : "
					+ task.getDeadline() + " priorirty : " + task.getPriority()
					+ "\n");
		}
		writer.flush();
		
	}
	
	
	@Override
	public void write(ITaskSet taskset) throws IOException {
		if (taskset.getAllTasksCount() == 0)
			return;
		List<ITask> set = new ArrayList<ITask>();
		for (ITask task : taskset) {
			set.add(task);
		}
		write(set);
	}
	
	
	
	
	@Override
	public void write(Iterable<ITask[]> tasksets) throws IOException {
		for (ITask[] set : tasksets) {
			write(set);
		}
		
	}

	@Override
	public void close() throws IOException {
			writer.close();
	}

	@Override
	public void flush() throws IOException {
		writer.flush();
	}

	

	

}
