package yartiss.engine.simulation.io.many;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.schedulable.GraphTask;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.simulation.system.schedulable.SubTask;

public class XmlTasksetsReader implements ITasksetsReader{

	
	private BlockingQueue<ITask[]> queue = new LinkedBlockingQueue<ITask []>(5);

	protected List<ITask> tasks;

	public XmlTasksetsReader(File file) throws Exception {
		this(new FileInputStream(file));
	}

	public XmlTasksetsReader(String fileName) throws Exception {
		this(new FileInputStream(fileName));
	}

	public XmlTasksetsReader(final InputStream input)	throws ParserConfigurationException, SAXException {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setNamespaceAware(true);
		final SAXParser parser = factory.newSAXParser();
		
		Thread parserThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					parser.parse(input, XmlTasksetsReader.this.getDefaultHandler());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		parserThread.start();
	}
	
	
	@Override
	public ITask[] nextTaskset() {
		ITask [] set = null;
		try {
			set = queue.take();
			if(set.length==0)
				set=null;
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return set;
	}

	@Override
	public List<ITask[]> readAll() {
		ITask [] set = null;
		List<ITask []> tasksets = new LinkedList<ITask[]>();

		while( (set = nextTaskset()) !=null){
			tasksets.add(set);
		}
		return tasksets;
	}

	@Override
	public void close() {
		
	}
	

	protected DefaultHandler getDefaultHandler() {
		return new DefaultHandler() {

			private ITask task;
			private List<SubTask> subtasks = new ArrayList<SubTask>();
			private String type;
			int nbTasks;
			private HashMap<String, Object> taskMap, subtaskMap;

			@Override
			public void startElement(String uri, String localName,	String qName, Attributes attributes) throws SAXException {

				if ("dataset".equals(localName)) {
					int nbTasksets = Integer.parseInt(attributes.getValue("nbTasksets"));
					//nbTasks = Integer.parseInt(attributes.getValue("nbTasks"));

				} else if ("taskset".equals(localName)) {
					// Do nothing for the moment
					tasks = new ArrayList<ITask>(nbTasks);

				} else if ("task".equals(localName)) {
					type = attributes.getValue("type");

					taskMap = new HashMap<String, Object>();
					taskMap.put("type", type);
					String param = type.toLowerCase();
					if("simple".equals(param)){

						taskMap.put("firstrelease",	attributes.getValue("firstRelease"));
						taskMap.put("wcet", attributes.getValue("wcet"));
						taskMap.put("wcee", attributes.getValue("wcee"));
						taskMap.put("period", attributes.getValue("period"));
						taskMap.put("deadline", attributes.getValue("deadline"));
						taskMap.put("priority", attributes.getValue("priority"));
						task = SchedulableFactory.newInstance(taskMap);
					}
					else if("graph".equals(param)){

						subtasks = new ArrayList<SubTask>(Integer.parseInt(attributes.getValue("nbSubtasks")));
						taskMap.put("firstrelease",attributes.getValue("firstRelease"));
						taskMap.put("period", attributes.getValue("period"));
						taskMap.put("deadline", attributes.getValue("deadline"));
						taskMap.put("priority", attributes.getValue("priority"));

					}
					else if("subtask".equals(param));
					else 
						throw new IllegalArgumentException(	"Illegal task type...");

				} 
				else if ("subtask".equals(localName)) {
					subtaskMap = new HashMap<String, Object>();
					subtaskMap.put("type", attributes.getValue("type"));
					subtaskMap.put("firstrelease",	attributes.getValue("firstRelease"));
					subtaskMap.put("wcee", "0");
					subtaskMap.put("wcet", attributes.getValue("wcet"));
					subtaskMap.put("period", attributes.getValue("period"));
					subtaskMap.put("deadline", attributes.getValue("deadline"));
					subtaskMap.put("priority", attributes.getValue("priority"));
					subtaskMap.put("nbproc", attributes.getValue("nbProc"));

					SubTask subtask = (SubTask) SchedulableFactory.newInstance(subtaskMap);
					subtask.setChildrenIndices(attributes.getValue("children").trim().split(" "));
					subtask.setId(Integer.parseInt(attributes.getValue("index")));
					subtasks.add(subtask);

				}
				super.startElement(uri, localName, qName, attributes);
			}

			@Override
			public void endElement(String uri, String localName, String qName)
					throws SAXException {

				if ("dataset".equals(localName)) {
					try {
						queue.put(new ITask[]{});
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				else if ("taskset".equals(localName)) {
					if (!tasks.isEmpty()) {
						ITask [] set = new ITask [tasks.size()];
						try {
							queue.put(tasks.toArray(set));
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				} else if ("task".equals(localName)) {
					if (!subtasks.isEmpty()) {
						taskMap.put("tasks", subtasks);

						task = SchedulableFactory.newInstance(taskMap);
						for (SubTask sub : ((GraphTask) task).getSubtasks()) {
							if (sub.getChildrenIndices() == null)
								continue;
							if (sub.getChildrenIndices().contains(-1))
								continue;
							for (SubTask sub2 : ((GraphTask) task)
									.getSubtasks()) {
								if ((sub.getChildrenIndices().contains(sub2
										.getId())) && !sub.equals(sub2)) {
									sub.addChildTask(sub2);
								}
							}
						}

						// TODO integrate children relations...
					}
					if (task != null) {
						tasks.add(task);

						// System.out.println("Task: " + task.toString());
						// GraphTask graph = (GraphTask) task;
						// for (SubTask subtask : graph.getSubtasks()) {
						//
						// System.out.println("***subtask: "
						// + subtask.toString());
						//
						// }
					}

				}
				super.endElement(uri, localName, qName);
			}

			@Override
			public void characters(char[] ch, int start, int length)
					throws SAXException {
				super.characters(ch, start, length);
			}

		};
		
	}
	
	public static void main(String[] args) throws Exception {
		XmlTasksetsReader reader = new XmlTasksetsReader(new File("file.xml"));
		
		ITask [] tasks = null;
		int count = 0;
		while( (tasks = reader.nextTaskset()) !=null){
			System.out.println(tasks);
			count++;
		}
		System.out.println("count="+count);
		
	}

}
