package yartiss.engine.simulation.io.one;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;

public class TextSimulationParametersReader implements ISimulationParametersReader{

	@Override
	public SimulationParameters read(File file) throws FileNotFoundException, IOException {
		return read(new FileInputStream(file));
	}

	@Override
	public SimulationParameters read(String file) throws FileNotFoundException , IOException {
		return read(new FileInputStream(file));
	}

	@Override
	public SimulationParameters read(InputStream in) throws  IOException{
		Scanner s = new Scanner(in);
		String firstLine = s.nextLine();
		List<Integer> parameters = null;
		String[] words = firstLine.split(" ");
		if (words.length > 1) {
			parameters = new ArrayList<Integer>();
			for (int i = 1; i < words.length; i++)
				parameters.add(Integer.parseInt(words[i]));
		}
		ISchedulingPolicy policy = SchedulingPolicyFactory
				.getInstance(words[0]);
		if (parameters != null)
			policy.setParameters(parameters);

		ITaskSet tasks = policy.createTaskSet();

		words = s.nextLine().split(" ");
		int simulationEnd = Integer.parseInt(words[0]);
		int nbProc = 1;
		if (words.length > 1)
			nbProc = Integer.parseInt(words[1]);
		String[] energyProfileLine = s.nextLine().split(" ");
		int[] energyParams = new int[energyProfileLine.length];
		for (int i = 0; i < energyParams.length; i++) {
			energyParams[i] = Integer.parseInt(energyProfileLine[i]);
		}
		IEnergyProfile energyProfile = EnergyProfileFactory.getInstance(
				"defaultEnergyProfile", energyParams);

		int nbTasks = s.nextInt();
		for (int i = 0; i < nbTasks; i++)
			if (!tasks.addTask(SchedulableFactory.newInstance("simple",
					s.nextInt(), s.nextInt(), s.nextInt(), s.nextInt(),
					s.nextInt(), s.nextInt())))
				System.err.println("Warning : two tasks with same priority");

		return new SimulationParameters(policy, simulationEnd, energyProfile,tasks, nbProc, true, true);
	}


}
