package yartiss.engine.simulation.io.one;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.system.ITask;

public class TextSimulationParametersWriter implements ISimulationParametersWriter{

	@Override
	public void write(SimulationParameters parameters, File file) throws FileNotFoundException, IOException {
			write(parameters, new FileOutputStream(file));
	}

	@Override
	public void write(SimulationParameters parameters, String file) throws FileNotFoundException, IOException {
		write(parameters, new FileOutputStream(file));
	}

	@Override
	public void write(SimulationParameters parameters, OutputStream file)throws IOException {
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(file));
		
		writer.write(parameters.getSchedulingPolicy().getPolicyName());
		
		if (parameters.getSchedulingPolicy().getParameters() != null) {
			for (Integer param : parameters.getSchedulingPolicy().getParameters()) {
				writer.write(" " + param);
			}
		}
		writer.write("\n" + parameters.getSimulationEnd() + " "	+ parameters.getNbProc() + "\n");

		writer.write(parameters.getEnergyProfile().getPowerRate() + " "
				+ parameters.getEnergyProfile().getMaxEnergyLevel() + " "
				+ parameters.getEnergyProfile().getMinEnergyLevel() + " "
				+ parameters.getEnergyProfile().getStartEnergyLevel() + "\n");

		writer.write(parameters.getTaskSet().getAllTasksCount() + "\n");

		for (ITask task : parameters.getTaskSet()) {
			writer.write(task.getFirstRelease() + " " + task.getWcet() + " "
					+ task.getWcee() + " " + task.getPeriod() + " "
					+ task.getDeadline() + " " + task.getPriority() + "\n");
		}

		writer.write("\n\n\n");
		writer.close();
	}

}
