package yartiss.engine.simulation.io.one;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;

public class XmlSimulationParametersWriter implements ISimulationParametersWriter{


	@Override
	public void write(SimulationParameters parameters, File file)throws FileNotFoundException, IOException {
		write(parameters, new FileOutputStream(file));

	}

	@Override
	public void write(SimulationParameters parameters, String file)	throws FileNotFoundException, IOException {
		write(parameters, new FileOutputStream(file));
	}

	@Override
	public void write(SimulationParameters parameters, OutputStream file)throws IOException {
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			Element mainElmt = document.createElement("dataset");
			document.appendChild(mainElmt);

			Element policyElmt = document.createElement("policy");
			policyElmt.setAttribute("name", parameters.getSchedulingPolicy().getPolicyName());

			
			List<Integer> policyParam = parameters.getSchedulingPolicy().getParameters();
			int nbParams= policyParam!=null ? policyParam.size() : 0;
			policyElmt.setAttribute("nbParams", Integer.toString(nbParams));
			if (nbParams > 0) {
				Element paramElmt;
				for (long param : parameters.getSchedulingPolicy().getParameters()) {
					paramElmt = document.createElement("parameter");
					paramElmt.setAttribute("name", "");
					paramElmt.setAttribute("value", Long.toString(param));
					policyElmt.appendChild(paramElmt);
				}

			}
			mainElmt.appendChild(policyElmt);

			Element simElmt = document.createElement("simulation");
			simElmt.setAttribute("endValue", parameters.getSimulationEnd()+"");

			simElmt.setAttribute("nbProc", parameters.getNbProc()+"");
			mainElmt.appendChild(simElmt);

			Element energyElmt = document.createElement("energyProfile");

			IEnergyProfile ep = parameters.getEnergyProfile();
			energyElmt.setAttribute("pr", ep.getPowerRate()+"");
			energyElmt.setAttribute("Emax", ep.getMaxEnergyLevel()+"");
			energyElmt.setAttribute("Emin", ep.getMinEnergyLevel()+"");
			energyElmt.setAttribute("E0", ep.getStartEnergyLevel()+"");

			mainElmt.appendChild(energyElmt);

			Element tasksElmt = document.createElement("tasks");

			ITaskSet taskSet = parameters.getTaskSet();

			tasksElmt.setAttribute("nbTasks", taskSet.getAllTasksCount()+"");
			tasksElmt.setAttribute("type", taskSet.getName());

			Element taskElmt;
			for (ITask task : taskSet) {
				taskElmt = document.createElement("task");
				taskElmt.setAttribute("type", task.getType());
				taskElmt.setAttribute("firstRelease", task.getFirstRelease()+"");
				taskElmt.setAttribute("wcet", task.getWcet()+"");
				taskElmt.setAttribute("wcee", task.getWcee()+"");
				taskElmt.setAttribute("period", task.getPeriod()+"");
				taskElmt.setAttribute("deadline", task.getDeadline()+"");
				taskElmt.setAttribute("priority", task.getPriority()+"");
				tasksElmt.appendChild(taskElmt);
			}

			mainElmt.appendChild(tasksElmt);

			TransformerFactory factory2 = TransformerFactory.newInstance();
			Transformer transformer = factory2.newTransformer();
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
			transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			DOMSource src = new DOMSource(document);
			StreamResult stream = new StreamResult(new OutputStreamWriter(file, "UTF-8"));
			transformer.transform(src, stream);
		}
		catch (Exception e) {
			e.printStackTrace();
			//throw new IOException(e.getLocalizedMessage());
		}
	}

	public void close() {


	}

}
