package yartiss.engine.simulation.scheduling.policies;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.scheduling.policies.dp.EDF;
import yartiss.engine.simulation.scheduling.policies.dp.EDFMultiProc;
import yartiss.engine.simulation.scheduling.policies.dp.EDL;
import yartiss.engine.simulation.scheduling.policies.dp.EDeg;
import yartiss.engine.simulation.scheduling.policies.dp.EHPDP2;
import yartiss.engine.simulation.scheduling.policies.dp.EHPDP5;
import yartiss.engine.simulation.scheduling.policies.dp.LaxityGraphMultiProc;
import yartiss.engine.simulation.scheduling.policies.dp.Mosse;
import yartiss.engine.simulation.scheduling.policies.dp.PDPASAP;
import yartiss.engine.simulation.scheduling.policies.dp.PDPST;
import yartiss.engine.simulation.scheduling.policies.fp.EHPFP2;
import yartiss.engine.simulation.scheduling.policies.fp.EHPFP3;
import yartiss.engine.simulation.scheduling.policies.fp.EHPFP5;
import yartiss.engine.simulation.scheduling.policies.fp.FP;
import yartiss.engine.simulation.scheduling.policies.fp.FPMultiProc;
import yartiss.engine.simulation.scheduling.policies.fp.FPeg;
import yartiss.engine.simulation.scheduling.policies.fp.PFPALAP;
import yartiss.engine.simulation.scheduling.policies.fp.PFPASAP;
import yartiss.engine.simulation.scheduling.policies.fp.PFPSlackTime;
import yartiss.engine.simulation.scheduling.policies.tools.DPSlackEnergyManager;
import yartiss.engine.simulation.scheduling.policies.tools.DPSlacktimeManager;
import yartiss.engine.simulation.scheduling.policies.tools.FPSlackEnergyManager1;
import yartiss.engine.simulation.scheduling.policies.tools.FPSlackTimeManager;

public class SchedulingPolicyFactory {
	
	private static final Map<String, ISchedulingPolicy> policies;
	private static List<ISchedulingPolicy> policiesLis = new ArrayList<ISchedulingPolicy>();
	
	static {
		policiesLis.add(new EDeg(new DPSlacktimeManager(), new DPSlackEnergyManager()));
		policiesLis.add(new Mosse());
		policiesLis.add(new EDF());
		policiesLis.add(new EDFMultiProc());
		policiesLis.add(new EDL(new DPSlacktimeManager()));
		policiesLis.add(new PDPASAP());
		policiesLis.add(new EHPDP2());
		policiesLis.add(new PDPST(new DPSlacktimeManager()));
		policiesLis.add(new EHPDP5(new DPSlacktimeManager()));
		
		policiesLis.add(new FP());
		policiesLis.add(new FP("SDM", "FP_DM"));
		policiesLis.add(new FP("SRM", "FP_RM"));
		policiesLis.add(new FPMultiProc());
		
		policiesLis.add(new PFPALAP());
		policiesLis.add(new PFPALAP("SDM", "PFP_ALAP_DM"));
		policiesLis.add(new PFPALAP("SRM", "PFP_ALAP_RM"));
		policiesLis.add(new PFPALAP("SECM", "PFP_ALAP_SECM"));
		policiesLis.add(new PFPALAP("SEDM", "PFP_ALAP_SEDM"));
		
		policiesLis.add(new PFPASAP());
		policiesLis.add(new PFPASAP("SDM", "PFP_ASAP_DM"));
		policiesLis.add(new PFPASAP("SRM", "PFP_ASAP_RM"));
		policiesLis.add(new PFPASAP("SECM", "PFP_ASAP_SECM"));
		policiesLis.add(new PFPASAP("SEDM", "PFP_ASAP_SEDM"));
		
		policiesLis.add(new FPeg(new FPSlackTimeManager(), new FPSlackEnergyManager1()));
		policiesLis.add(new EHPFP2());
		policiesLis.add(new EHPFP3(new FPSlackTimeManager()));
		
		policiesLis.add(new PFPSlackTime(new FPSlackTimeManager()));
		policiesLis.add(new PFPSlackTime("SDM", "PFP_ST_DM", new FPSlackTimeManager()));
		policiesLis.add(new PFPSlackTime("SRM", "PFP_ST_RM", new FPSlackTimeManager()));
		policiesLis.add(new PFPSlackTime("SECM", "PFP_ST_SECM", new FPSlackTimeManager()));
		policiesLis.add(new PFPSlackTime("SEDM", "PFP_ST_SEDM", new FPSlackTimeManager()));
		
		policiesLis.add(new EHPFP5(new FPSlackTimeManager()));
		policiesLis.add(new LaxityGraphMultiProc());
		
		policiesLis.add(new PFPASAP());

		
		policies = new HashMap<String, ISchedulingPolicy>();
		
		for (ISchedulingPolicy policy : policiesLis) {
			policies.put(policy.getPolicyName(), policy);
		}
	}
	
	public static void registerPolicy(ISchedulingPolicy policy){
		policies.put(policy.getPolicyName(), policy);
	}
	
	public static ISchedulingPolicy getInstance(String polycyName) {
		if(policies.containsKey(polycyName)){
			return policies.get(polycyName).newInstance();
		}
		throw new IllegalArgumentException(polycyName);
	}

	public static String[] getAvailableSchedulingPoliciesNames() {
		String [] availible = new String [policies.size()];
		int i=0;
		for (String policy : policies.keySet()) {
			availible[i++] = policy;
		}
		Arrays.sort(availible);
		return availible;
	}
}
