package yartiss.engine.simulation.scheduling.policies.dp;

import java.util.ArrayList;
import java.util.SortedSet;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.AbstractMonoProcessorSchedulingPolicy;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.PeriodicTask;

public class Mosse extends AbstractMonoProcessorSchedulingPolicy {

	
	public Mosse(String tasksetType, String policyName) {
		super(tasksetType, policyName);
		policyName = "Mosse";
		tasksetType = "SIECRM";
	}
	
	public Mosse(){
		this("SIECRM","Mosse");
	}
	
	int replenish = 1, dicipate=-1; 
	private long maxIdle = 0;
	private long remainingIdle = 0, idleStartDate=0; 
	private boolean done = false, idle=false;
	private int state = dicipate;
	private long cycle = 0;
	private ITask lamdatask = null;
	
	private boolean debug = false;
	@Override
	public ITask chooseNextTask(ITaskSet taskSet, IEnergyProfile energyProfile,	long date, EventGenerator evGen) {
		long lRl=0, lDl=0, pi=0;
		if(!done){
			for (ITask task : taskSet) {
				cycle = task.getPeriod();
				lamdatask = task;
				pi = energyProfile.getPowerRate() - task.getWcee()/task.getWcet();
				if( pi >= 0)lRl += pi*task.getWcet(); 
				else 	    lDl += -pi*task.getWcet(); 
			}
			maxIdle = Math.round(Math.ceil(lDl - lRl)/((double)energyProfile.getPowerRate()));
			remainingIdle = maxIdle;
			done = true;
			//if(debug)System.out.println(lDl +" > "+ lRl);
		}
		
		if(date%cycle ==0 && !idle){
			remainingIdle = maxIdle;
			if(remainingIdle >0 && energyProfile.getCurrentEnergyLevel()+energyProfile.getPowerRate() <= energyProfile.getMaxEnergyLevel() ){
				state = replenish;
				idleStartDate = date;
				idle = true;
				evGen.generateEvent("check_energy_state", lamdatask, date + remainingIdle, null);
				//if(debug)System.out.println(date+" : "+lDl +" > "+ lRl+" state="+state+" wakeup="+remainingIdle);
				return null;
			}
		}
		else if(remainingIdle > 0 && idle){
			long diff = date - idleStartDate;
			remainingIdle -= diff;
			if(diff>0)
				idle = false;
			else 
				return null;
		}
		
		SortedSet<ITask> activeTasks = taskSet.getActiveTasks(date);
		if(!activeTasks.isEmpty()){
			
			ArrayList<ITask> R = new ArrayList<ITask>();
			ArrayList<ITask> D = new ArrayList<ITask>();
			for (ITask task : activeTasks) {
				pi = energyProfile.getPowerRate() - task.getWcee()/task.getWcet();
				if( pi >= 0)R.add(task);
				else D.add(task);
			}
			
			if( remainingIdle<=0 && R.isEmpty()){
				state=dicipate;
				//if(debug)System.out.println(date +" : replenish==>decipate  A");
			}
			
			if((energyProfile.getCurrentEnergyLevel()+energyProfile.getPowerRate()) >= energyProfile.getMaxEnergyLevel() ){
				state = dicipate;
				//if(debug)System.out.println(date +" : replenish==>decipate  B");
			}
			

			if( state==dicipate && !D.isEmpty()){
				long hlcet = energyProfile.howLongCanExecute(D.get(0));
				if (hlcet > 0) {
					state=dicipate;
					if (hlcet < D.get(0).getRemainingCost())
						evGen.generateEvent("check_energy_state", D.get(0), date + hlcet, null);
					//if(debug)System.out.println(date+" : descipate : t"+D.get(0).getId()+" E="+energyProfile.getCurrentEnergyLevel()+"   state="+(state==dicipate)+" load="+load);
					return D.get(0);
				}
				if(debug)System.out.println(date +" : decipate==>replenish A");
				state = replenish;
			}
			else if( state==dicipate){
				state = replenish;
				//if(debug)System.out.println(date +" : decipate==>replenish B");
			}

			if( state==replenish && !R.isEmpty()){
				long hlcet = energyProfile.howLongCanExecute(R.get(0));
				if (hlcet > 0) {
					if (hlcet < R.get(0).getRemainingCost())
						evGen.generateEvent("check_energy_state", R.get(0), date + hlcet, null);
					//if(debug)System.out.println(date+" : replenish: t"+R.get(0).getId()+" E="+energyProfile.getCurrentEnergyLevel()+"   state="+(state==replenish)+" load="+load);
					return R.get(0);
				}
			}
			
//			lDl > lRl
			state = replenish;
			long wakeup = Math.max(1, remainingIdle);
			idleStartDate = date;
			idle = true;
			evGen.generateEvent("check_energy_state", activeTasks.first(), date + wakeup, null);
			//if(debug)System.out.println(date+" : "+lDl +" > "+ lRl+" state="+state+" wakeup="+wakeup+" remainingIdle="+remainingIdle);
			return null;
		}
		return  null;
	}
	
	@Override
	public ISchedulingPolicy newInstance() {
		return new Mosse(tasksetType, policyName);
	}
}