package yartiss.engine.simulation.scheduling.policies.fp;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.AbstractMonoProcessorSchedulingPolicy;
import yartiss.engine.simulation.scheduling.policies.tools.FPSlackTimeManager;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;

public class PFPALAP extends AbstractMonoProcessorSchedulingPolicy{
	
	private FPSlackTimeManager slackTimeManager = new FPSlackTimeManager();
	public PFPALAP(String tasksetType, String policyName) {
		super(tasksetType, policyName);
	}
	
	public PFPALAP(){
		this("priority", "PFP_ALAP");
	}

	
	@Override
	public ISchedulingPolicy newInstance() {
		return new PFPALAP(tasksetType, policyName);
	}

	private Map<String, Long> metrics = new HashMap<String, Long>();
	long wakeup = -1;
	
	AbstractMonoProcessorSchedulingPolicy p = new FP();
	
	@Override
	public ITask chooseNextTask(ITaskSet taskSet, IEnergyProfile energyProfile,	long date, EventGenerator evGen) {
		
//		if(date>=19)
//			return p.chooseNextTask(taskSet, energyProfile, date, evGen);
		metrics.clear();
		SortedSet<ITask> activeTasks = taskSet.getActiveTasks(date);
		if(!activeTasks.isEmpty()){
			ITask first = activeTasks.first();
			long st = slackTimeManager.getSlackTime(date, taskSet);
			metrics.put(Event.SlackTime, st);	
			//System.out.println(date +" : slack time = "+st );
			if(st>0){
				evGen.generateEvent("check_energy_state", activeTasks.first(), date + st, null);
				wakeup = date + st;
				return null;
			}
			long hlcet = energyProfile.howLongCanExecute(first);
			if (hlcet <= 0) {
				evGen.generateEvent("energy_failure", activeTasks.first(), date, null);
				return null;
			}
			if (hlcet < activeTasks.first().getRemainingCost())
				evGen.generateEvent("check_energy_state", activeTasks.first(), date + hlcet, null);
			return activeTasks.first();
		}
		return null;
	}
}
