package yartiss.engine.simulation.scheduling.policies.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;
import java.util.TreeSet;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.io.many.ITasksetsReader;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.io.one.SimulationParametersIOFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.SystemMetrics;
import yartiss.engine.simulation.system.schedulable.SimpleTask;
import yartiss.engine.simulation.system.taskset.FixedPriorityTaskSet;

public class EnergyAwareFeasibilityTest {

	private static int simEnd = 10000;

	static class Entryy{
		long  t;
		long c;
		long e;
		ITask task;
		public Entryy(long t, ITask task) {
			super();
			this.t = t;
			this.task = task;
			c = task.getWcet();
			e = task.getWcee();
		}
	}

	public static void main(String[] args) throws Exception {
		String source1 = "./src/test/resources/hidden_systems/400000_systems_pr15.txt";
		String source2 = "./src/test/resources/hidden_systems/20000_systems_pr15.txt";

		MonTest();

		//Test();
		//CompareBornes(source2);
		//TestFeasibilityTest(source2);
	}


	public static void MonTest() throws FileNotFoundException, IOException, Exception{

		String src = "C:/Users/Younès/Desktop/toto.xml";
		SimulationParameters param = SimulationParametersIOFactory.newSimulationParametersReaderInstance("xml").read(src);
		System.out.println(param.getTaskSet().getAllTasksCount());
		System.out.println(testFeasibility(param.getTaskSet(),param.getEnergyProfile()));
	}


	private static TreeSet<Entryy> getB(ITaskSet set, ITask task){
		TreeSet<Entryy> sorted = new TreeSet<Entryy>(new Comparator<Entryy>() {
			@Override public int compare(Entryy o1, Entryy o2) {
				long comp = o1.t - o2.t;
				if(comp==0)
					return o1.task.getPriority() - o2.task.getPriority();
				return (int)comp;
			}
		});
		for (ITask itache : set.getHeadSet(task, true)) {
			long t=0;

			do{
				//System.out.println("t="+t);
				sorted.add(new Entryy(t, itache));
				t = itache.getNextActivation(t);

			}while(t < task.getDeadline());
			//System.out.println("-------------");

		}
		System.out.print("[");
		for (Entryy entryy : sorted) {
			System.out.print("(t"+entryy.task.getId()+","+entryy.t+","+entryy.c+")");
		}
		System.out.println("]");
		return sorted;
	}

	private static long ceil(long d, long n){
		double dd = d;
		double nn = n;
		return Math.round(Math.ceil(dd/nn));
	}


	public static boolean testFeasibility(ITaskSet set, IEnergyProfile ep){


		for (ITask task : set) {
			long E = ep.getStartEnergyLevel();
			long pr = ep.getPowerRate();
			long Emax = ep.getMaxEnergyLevel();
			TreeSet<Entryy> sorted = getB(set, task);
			Stack<Entryy> pile = new Stack<Entryy>();
			long responseTime = 0;
			Entryy firstpile = null;
			while(!sorted.isEmpty()){
				long diff = 0;
				Entryy first = sorted.pollFirst();
				System.out.println(first.t+" : t"+first.task.getId()+"---------");
				Entryy next = sorted.isEmpty() ? null : sorted.first();
				if(next != null){
					diff = next.t - responseTime;
					diff = diff<0 ? 0 : diff;
					//System.out.println("diff = "+next.t+"-"+responseTime+" = "+diff+" >= "+first.c);
				}
				long spend = Math.max(ceil(first.e-E,pr), first.c);

				if(diff==0 || diff>=spend){
					responseTime += spend;
					diff -= spend;
					E += pr*spend-first.e;
					first.c=0;
					first.e=0;

					while(diff>0 && !pile.isEmpty()){
						firstpile = pile.pop();
						spend = Math.max(ceil(firstpile.e-E,pr), firstpile.c);
						diff-= spend;
						responseTime +=spend;
						E += spend*pr-firstpile.e;
						if(diff<0){	
							spend = -diff;
							long exec = (spend*pr)/(firstpile.task.getWcee()/firstpile.task.getWcet());
							firstpile.c -= exec;
							firstpile.e -= exec*firstpile.task.getWcee()/firstpile.task.getWcet();//TODO
							pile.push(firstpile);
							responseTime += diff;
							E -= spend*pr-firstpile.e;
							//System.out.println("1 r="+responseTime+"   E="+E);
						}
						else {
							firstpile.c = 0;
							firstpile.e = 0;
							if(firstpile.task==task){
								System.out.println("c la");
								break;
							}
						}
						System.out.println("2==> r="+responseTime+"   E="+E);
					}
					System.out.println("==>3 r="+responseTime+"   E="+E);
				}
				else {
					E += spend*pr-first.e;
					spend -= diff;
					long exec = (spend*pr)/(first.task.getWcee()/first.task.getWcet());
					first.c -= exec;
					first.e -= exec*first.task.getWcee()/first.task.getWcet();//TODO
					pile.push(first);
					responseTime += diff;
					E -= spend*pr-first.e;
					//System.out.println("4 r="+responseTime);
				}
				System.out.println("r = "+responseTime);
				if(responseTime > task.getDeadline()){
					System.out.println("task t="+task.getId()+" n'est pas fiasable");
					return false;
				}
				else if((first.task == task && first.c==0) || (firstpile!=null && firstpile.task == task && firstpile.c==0)){
					System.out.println("c ici ="+first.c);
					break;
				}
			}
			System.out.println("task t="+task.getId()+" responsetime="+responseTime+" E="+E);
		}
		return true;
	}


	public static void TestFeasibilityTest(String systemsSource) throws Exception{
		File file = new File(systemsSource);
		long [] prs = new long []{15};//,20,25,30,35};

		System.out.println(file.getAbsolutePath()+"	 : "+file.canRead());

		ITask[] tasks = null;

		for (long pr : prs) {
			ITasksetsReader reader = TasksetsIOFactory.newTasksetsReaderInstance("txt", file);
			int countFailues=0;
			int count = 0;
			double rate = 0;
			while((tasks = reader.nextTaskset()) != null){

				ISchedulingPolicy policy = SchedulingPolicyFactory.getInstance("FPL");
				ITaskSet set = policy.createTaskSet();
				for (ITask task : tasks) {
					set.addTask(task);
				}

				long studyPeriod = SystemMetrics.hyperPeriod(set);

				if(studyPeriod < 1000){
					count ++;
					EnergyAwareFeasibilityTest test = new EnergyAwareFeasibilityTest();
					boolean feasibileSP = test.FP_ALAP_FeasibilityTest(set, pr, false);
					test = new EnergyAwareFeasibilityTest();
					boolean feasibileHP = test.FP_ALAP_FeasibilityTest(set, pr, true);

					IEnergyProfile ep = EnergyProfileFactory.getInstance("default", 15, Integer.MAX_VALUE, 0, 0);
					Simulation sim = new Simulation(new SimulationParameters(policy, simEnd, ep, set, 1, true, true));
					sim.simulate();

					//					if((feasibileSP || feasibileHP || sim.getStatistics().isFinished())   &&  !(feasibileSP && feasibileHP && sim.getStatistics().isFinished())){
					//						System.out.println("feasibileSP="+feasibileSP+"   feasibileHP="+feasibileHP+"  sim="+sim.getStatistics().isFinished());
					//						for (ITask task : set) {
					//							System.out.println(task);
					//						}
					//						System.out.println("----------------------------------------------------------");
					//						countFailues ++;
					//					}
				}
			}
			reader.close();
			rate = countFailues*100/count;
			System.out.println("pr "+pr+" : "+countFailues+" rate="+rate);
		}
	}

	public static void CompareBornes(String systemsSource) throws Exception{
		File file = new File(systemsSource);
		long [] prs = new long []{15,20,25,30,35};

		System.out.println(file.canRead());
		System.out.println(file.getAbsolutePath());



		ITask[] tasks = null;

		for (long pr : prs) {
			ITasksetsReader reader = TasksetsIOFactory.newTasksetsReaderInstance("txt", file);
			int count=0;
			double rate = 0;
			while((tasks = reader.nextTaskset()) != null){

				ITaskSet set = new FixedPriorityTaskSet();
				for (ITask task : tasks) {
					set.addTask(task);
				}

				long studyPeriod = SystemMetrics.hyperPeriod(set);
				//System.out.println("studyPeriod="+studyPeriod);

				if(studyPeriod < 1000){
					count++;
					//System.out.println(count);
					EnergyAwareFeasibilityTest test = new EnergyAwareFeasibilityTest();
					boolean feasibile = test.FP_ALAP_FeasibilityTest(set, pr, true);
					if(feasibile){
						long maxBorne = borneMax(set,pr);
						long realBorne = test.Emax;
						rate += ((realBorne*100)/((double)maxBorne));
					}
				}
			}
			reader.close();
			rate /= count;
			System.out.println("pr "+pr+" : "+rate);
		}
	}

	public static void Test(){
		long pr = 3;
		ITaskSet set = new FixedPriorityTaskSet();
		set.addTask(new SimpleTask(0, 2,  8,  8,  3, 1, null));
		set.addTask(new SimpleTask(0, 2, 10, 10,  9, 2, null));
		set.addTask(new SimpleTask(0, 2, 10, 16, 16, 3, null));
		//set.addTask(new SimpleTask(firstRelease, wcet, wcee, period, deadline, priority, p));
		EnergyAwareFeasibilityTest test = new EnergyAwareFeasibilityTest();

		System.out.println(test.FP_ALAP_FeasibilityTest(set, pr, true));
		System.out.println("Emax="+test.Emax);
		System.out.println("borneMax="+borneMax(set, pr));
	}


	private long slackTime_i_t;
	private long busyPeriod_i_t;
	private long energyDemand_i_t;
	public long Emax;

	public boolean FP_ALAP_FeasibilityTest(ITaskSet taskSet, long pr, boolean isUntilHP){
		long slacktime=0, busyPeriod=0,energyDemand=0;

		long t=0;
		long E=0;
		ITask lowestPriorityTask = getLowestPriority(taskSet);
		long studyPeriod = isUntilHP ? SystemMetrics.hyperPeriod(taskSet) : SystemMetrics.effectiveDeadline(lowestPriorityTask, taskSet, lowestPriorityTask.getDeadline());

		if( isUntilHP ) System.out.println("hyper period = "+studyPeriod);
		else System.out.println("study period ="+studyPeriod);

		Emax = 0;
		ITask tau_k = null;
		do{
			slacktime = Long.MAX_VALUE;
			updateRemainingCost(taskSet,tau_k, t);
			for (ITask tau_i : taskSet) {
				computeTaskSlackTime(t, tau_i, taskSet);
				if(t + slackTime_i_t + busyPeriod_i_t > getNextDeadline(tau_i, t, true)){
					//System.out.println(t+" : task tau_"+tau_i.getId()+" : deadline missed : "+(t + slackTime_i_t + busyPeriod_i_t)+" > "+tau_i.getNextDeadline(t, true));
					return false;
				}
				if(slackTime_i_t < slacktime){
					tau_k = tau_i;
					slacktime = slackTime_i_t;
					busyPeriod = busyPeriod_i_t;
					energyDemand = energyDemand_i_t;
				}
			}

			executeBP(taskSet, tau_k, t, t + slacktime + busyPeriod);

			//if(!isUntilHP)
			System.out.println(t+" : ST="+slacktime+" BP="+busyPeriod+" ED="+energyDemand);

			E += pr * slacktime; 
			if(E>Emax)
				Emax = E;
			E+= pr * +busyPeriod;
			if(E < energyDemand ){
				//System.out.println(t+" : energy failure");
				return false;
			}



			//update remaining cost of jobs activated between t and t+slacktime+busyperiod and having priority lower than tau_k  
			for (ITask task : taskSet) {
				if(task.getPriority() > tau_k.getPriority()){
					long na = task.getNextActivation(t);
					if(na < t+(slacktime + busyPeriod)){
						setRemainingCost(task, task.getWcet());
						setRemainingEnergyCost(task, task.getWcee());
					}
				}

			}

			E -= energyDemand;
			t += (slacktime + busyPeriod);
			//if(!isUntilHP)
			System.out.println("------------------------------------------- t="+t+" E="+E+" Emax="+Emax);

		}while(t < studyPeriod);
		return true;
	}

	private void executeBP(ITaskSet tasks, ITask tau_i, long from, long to) {
		for (ITask task : tasks.getHeadSet(tau_i, true)) {			
			setRemainingCost(task, 0);
			setRemainingEnergyCost(task, 0);
		}
	}

	private void updateRemainingCost(ITaskSet taskSet,ITask tau_k, long t) {
		for (ITask task : taskSet) {
			if(SystemMetrics.effectiveDeadline(task, taskSet, getNextDeadline(task, t-1, false)) == t){
				setRemainingCost(task,0);
				setRemainingEnergyCost(task, 0);
			}
			if(t % task.getPeriod() == 0 ){
				setRemainingCost(task, task.getWcet());
				setRemainingEnergyCost(task, task.getWcee());
			}
		}
	}

	private void computeTaskSlackTime(long date, ITask task_i, ITaskSet tasks) {
		slackTime_i_t = 0;
		busyPeriod_i_t = 0;
		long w_m1 = date;

		long di = SystemMetrics.effectiveDeadline(task_i, tasks, getNextDeadline(task_i, date,true)); // verifier que si la tache a null et/ou le jobid etc.
		long bp_i = 0, ed_i=0;
		long w_m = -1;
		//long time = -1;
		while (w_m1 <= di) {

			w_m = w_m1;
			bp_i = getWorkLoadUntil(date, w_m, task_i, tasks);
			ed_i = getEnergyLoadUntil(date, w_m, task_i, tasks);
			w_m1 = date + slackTime_i_t + bp_i;
			if (w_m1 == w_m) {
				//if(date==time)System.out.println("			 >>>>");
				long nu = getIdlePeriodLength(date, w_m, task_i, tasks);
				slackTime_i_t += nu;
				busyPeriod_i_t = bp_i;

				energyDemand_i_t = ed_i;
				w_m1 += nu + 1;
			}
			//if(date==time)System.out.println("			 ST_"+task_i.getId()+"="+slackTime_i_t+" BP_"+task_i.getId()+"="+bp_i+" w_m="+w_m+" w_m1="+w_m1+" d_i="+di);
		}

		//System.out.println("-----"+date+" ST_"+task_i.getId()+"="+slackTime_i_t+" BP_"+task_i.getId()+"="+busyPeriod_i_t+" ED_"+task_i.getId()+"="+energyDemand_i_t+" rc="+getRemainingCost(task_i));
	}

	/**
	 * Compute the total workload between date and w
	 * 
	 * @param w
	 * @param task
	 * @return the total workload until time w
	 */
	private long getWorkLoadUntil(long systemDate, long w, ITask task, ITaskSet tasks) {

		long inter = 0;
		for (ITask t : tasks.getHeadSet(task, true)) {			
			inter += getRemainingCost(t) + Math.ceil(Math.max(w - t.getNextActivation(systemDate), 0) / (double) t.getPeriod()) * t.getWcet();
		}

		return inter;
	}


	private long getEnergyLoadUntil(long systemDate, long w, ITask task, ITaskSet tasks) {

		long inter = 0;
		for (ITask t : tasks.getHeadSet(task, true)) {			
			inter += getRemainingEnergyCost(t) + Math.ceil(Math.max(w - t.getNextActivation(systemDate), 0) / (double) t.getPeriod()) * t.getWcee();
		}

		return inter;
	}

	private long getIdlePeriodLength(long currentDate, long w_m, ITask task, ITaskSet tasks) {

		long di_t = getNextDeadline(task, currentDate, true);
		long idle = Math.min(Math.max(0, di_t - w_m), getIdleRightPart(currentDate, w_m, task, tasks));

		return idle;

	}

	private long getIdleRightPart(long currentDate, long w, ITask task, ITaskSet tasks) {

		long value = Long.MAX_VALUE;
		for (ITask t : tasks.getHeadSet(task, true)) {
			value = Math.min(
					value,
					(long) Math.max(0, Math.ceil((w - t.getNextActivation(currentDate)) / (double) t.getPeriod())) * t.getPeriod() + t.getNextActivation(currentDate) - w);
			//			if(currentDate == 19)
			//				System.out.println("				t"+t.getId()+" : "+value+" actv="+ t.getNextActivation(currentDate)+" w="+w+"	"+(Math.ceil((w - t.getNextActivation(currentDate)) / (double) t.getPeriod())*t.getPeriod())+" + ("+t.getNextActivation(currentDate)+" - "+w+")");

		}
		return value;

	}

	// borne ---------------------------------------------------------
	public static long borneMax(ITaskSet set, long pr){
		long hyperPeriod = SystemMetrics.hyperPeriod(set);
		double utilization = SystemMetrics.utilisation(set);
		double borne = (1-utilization)*hyperPeriod;
		return (Math.round(borne)+1) * pr;
	}

	public static long borneMin(ITaskSet set, long pr){
		long borneMax = borneMax(set, pr);
		long MLBPEnergyDemand = getMostLongerBusyPeriodEnergyDemand(set);

		while(borneMax > MLBPEnergyDemand*2){
			borneMax -= MLBPEnergyDemand;
		}

		return borneMax;
	}

	//utility ---------------------------------------------------------------------------------------------

	private static long getMostLongerBusyPeriodEnergyDemand(ITaskSet set) {
		// TODO Auto-generated method stub
		return 0;
	}

	private Map<ITask, Long> remainingCost = new HashMap<ITask, Long>();

	private long getRemainingCost(ITask task){
		if(remainingCost.containsKey(task))
			return remainingCost.get(task);
		remainingCost.put(task, Long.valueOf(0));
		return 0;
	}

	private void setRemainingCost(ITask task, long newValie){
		remainingCost.put(task, Long.valueOf(newValie));
	}

	private Map<ITask, Long> remainingEnergyCost = new HashMap<ITask, Long>();
	private long getRemainingEnergyCost(ITask task){
		if(remainingEnergyCost.containsKey(task))
			return remainingEnergyCost.get(task);
		remainingEnergyCost.put(task, Long.valueOf(0));
		return 0;
	}

	private void setRemainingEnergyCost(ITask task, long newValie){
		remainingEnergyCost.put(task, Long.valueOf(newValie));
	}




	private long getNextDeadline(ITask task, long date, boolean nextPeriod){
		if (date < task.getFirstRelease())
			return task.getFirstRelease() + task.getDeadline();

		long currentPeriod = (date - task.getFirstRelease()) / task.getPeriod();

		if(nextPeriod && getRemainingCost(task)==0)
			currentPeriod++;

		return task.getFirstRelease() + currentPeriod * task.getPeriod() + task.getDeadline();
	}

	private ITask getLowestPriority(ITaskSet set){
		ITask task = null;
		for (ITask tau_i : set) {
			if(task == null || task.getPriority() < tau_i.getPriority())
				task = tau_i;
		}
		return task;
	}

	/*
	/home/algotr/workspace/YARTISS/yartiss-engine/./src/test/resources/hidden_systems/20000_systems_pr15.txt	 : true
	feasibileSP=true   feasibileHP=false  sim=false
	[4849]First release=0	WCET=4	WCEE=216	T=32	D=16	P=13
	[4850]First release=0	WCET=1	WCEE=48		T=48	D=32	P=14
	[4851]First release=0	WCET=1	WCEE=16		T=48	D=22	P=15
	[4852]First release=0	WCET=3	WCEE=186	T=40	D=32	P=16
	----------------------------------------------------------------
	feasibileSP=false   feasibileHP=false  sim=true
	[24889]First release=0	WCET=3	WCEE=60		T=30	D=26	P=77
	[24890]First release=0	WCET=3	WCEE=54		T=25	D=12	P=78
	[24891]First release=0	WCET=2	WCEE=42		T=25	D=16	P=79
	[24892]First release=0	WCET=13	WCEE=208	T=90	D=31	P=80
	---------------------------------------------------------------
	feasibileSP=false   feasibileHP=false  sim=true
	[27505]First release=0	WCET=2	WCEE=70		T=40	D=21	P=81
	[27506]First release=0	WCET=2	WCEE=68		T=20	D=20	P=82
	[27507]First release=0	WCET=5	WCEE=125	T=22	D=19	P=83
	[27508]First release=0	WCET=3	WCEE=84		T=44	D=44	P=84
	----------------------------------------------------------------
	feasibileSP=false   feasibileHP=false  sim=true
	[27869]First release=0	WCET=1	WCEE=16		T=32	D=30	P=37
	[27870]First release=0	WCET=1	WCEE=16		T=20	D=8		P=38
	[27871]First release=0	WCET=9	WCEE=315	T=30	D=26	P=39
	[27872]First release=0	WCET=2	WCEE=68		T=40	D=40	P=40
	---------------------------------------------------------------
	feasibileSP=true   feasibileHP=false  sim=true
	[73685]First release=0	WCET=1	WCEE=16		T=24	D=5	P=5
	[73686]First release=0	WCET=16	WCEE=256	T=24	D=20	P=6
	[73687]First release=0	WCET=9	WCEE=144	T=120	D=77	P=7
	[73688]First release=0	WCET=18	WCEE=288	T=160	D=123	P=8
	---------------------------------------------------------------
	feasibileSP=true   feasibileHP=false  sim=true
	[76089]First release=0	WCET=14	WCEE=224	T=25	D=17	P=33
	[76090]First release=0	WCET=10	WCEE=160	T=75	D=72	P=34
	[76091]First release=0	WCET=3	WCEE=48		T=90	D=85	P=35
	[76092]First release=0	WCET=15	WCEE=240	T=90	D=84	P=36
	----------------------------------------------------------------
	feasibileSP=true   feasibileHP=false  sim=true
	[76829]First release=0	WCET=2	WCEE=32		T=48	D=24	P=5
	[76830]First release=0	WCET=4	WCEE=64		T=96	D=88	P=6
	[76831]First release=0	WCET=69	WCEE=1104	T=96	D=84	P=7
	[76832]First release=0	WCET=12	WCEE=192	T=128	D=109	P=8
	---------------------------------------------------------------
	pr 15 : 7 rate=0.0
	 */

}
