package yartiss.engine.simulation.statistics;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.IStatisticCriterion;
import yartiss.engine.simulation.events.CheckEffectiveDeadlineEvent;
import yartiss.engine.simulation.events.DeadlineMissedEvent;
import yartiss.engine.simulation.events.EnergyFailureEvent;
import yartiss.engine.simulation.events.EventVisitorAdapter;
import yartiss.engine.simulation.events.IEventVisitor;
import yartiss.engine.simulation.events.JobExecutionBeginEvent;
import yartiss.engine.simulation.events.JobExecutionEndEvent;

public class AverageBusyPeriodStatisticCretirion extends AbstractStatisticCriterion {

	private long nbBP = 0;
	private double sumBP= 0 ;
	private long currentBP=0;
	private boolean bpEndDetected = false, bpBeginDetected = false;
	private Event currentEvent = null;
	private long oldDate=0;
	
	@Override
	public void simulationStarted() {
		nbBP = 0; 
		sumBP= 0; 
		currentBP=0;
		bpEndDetected = false; 
		bpBeginDetected = false; 
		currentEvent = null;
		oldDate = 0;
	}

	@Override
	public void simulationEnd(long date) {
		

	}

	@Override
	public void nextEventReady(Event event) {
		if(currentEvent != null)
			oldDate = currentEvent.getDate();
		currentEvent = event;
	}

	@Override
	public void eventPerformed(Event event) {
		super.eventPerformed(event);
		currentEvent.accept(visitor);
		if(currentEvent.getDate()!=oldDate){
			if( bpEndDetected && !bpBeginDetected){    // parameters.getTaskSet().getActiveTasks(e.getDate()).isEmpty()
				sumBP += currentBP;
				currentBP=0;
				nbBP++;
			}
			bpEndDetected = false;
			bpBeginDetected = false;
		}

	}
	
	Event lastBegin = null;
	Event lastEnd = null;
	IEventVisitor visitor = new EventVisitorAdapter() {

		@Override	public void visit(EnergyFailureEvent   e) { end(e); }
		@Override	public void visit(JobExecutionEndEvent e) {	end(e);	}
		@Override	public void visit(DeadlineMissedEvent  e) {	end(e);	}
		@Override	public void visit(CheckEffectiveDeadlineEvent e) {  }
		@Override
		public void visit(JobExecutionBeginEvent e) {
			bpBeginDetected = true;
			lastBegin = e;
		}

		private void end(Event e){
			bpEndDetected = true;
			bpBeginDetected = false;
			if(lastBegin != null)
				currentBP += (e.getDate() - lastBegin.getDate());
			lastEnd = e;
		}

	};
	

	@Override
	public String getName() {
		return "Average busy-period";
	}

	@Override
	public IStatisticCriterion newInstance() {
		return new AverageBusyPeriodStatisticCretirion();
	}

	@Override
	public Object getValue() {
		return  nbBP>0 ? sumBP/nbBP : 0;
	}

	@Override
	public Object getAggregatedValue() {
		return aggregation.getPercent(aggregation.getAverage(), parameters.getSimulationEnd());
	}

}
