/**
 * 
 */
package yartiss.engine.simulation.statistics;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.IStatisticCriterion;
import yartiss.engine.simulation.events.CheckEffectiveDeadlineEvent;
import yartiss.engine.simulation.events.DeadlineMissedEvent;
import yartiss.engine.simulation.events.EnergyFailureEvent;
import yartiss.engine.simulation.events.EventVisitorAdapter;
import yartiss.engine.simulation.events.IEventVisitor;
import yartiss.engine.simulation.events.JobExecutionBeginEvent;
import yartiss.engine.simulation.events.JobExecutionEndEvent;

/**
 * @author Younes
 *
 */
public class AverageIdlePeriodStatisticCretirion extends AbstractStatisticCriterion {

	long  nbIP = 0;
	double sumIP= 0;
	boolean ipBeginDetected=true;
	private Event currentEvent=null;
	
	@Override
	public void simulationStarted() {
		nbIP = 0;
		sumIP= 0;
		ipBeginDetected=true;
		currentEvent=null;
	}

	@Override
	public void simulationEnd(long date) {
		if(ipBeginDetected){
			long ipBegin = lastEnd != null ? lastEnd.getDate() : 0;
			long length = (date>parameters.getSimulationEnd() ? parameters.getSimulationEnd() : date) - ipBegin;
			if(length>0){
				sumIP += length;
				nbIP++;
			}
		}

	}

	@Override
	public void nextEventReady(Event event) {
		currentEvent = event;
	}

	@Override
	public void eventPerformed(Event event) {
		super.eventPerformed(event);
		currentEvent.accept(visitor);
	}

	@Override
	public String getName() {
		return "Average idle-period";
	}

	@Override
	public IStatisticCriterion newInstance() {
		return new AverageIdlePeriodStatisticCretirion();
	}

	@Override
	public Object getValue() {
		return  nbIP>0 ? sumIP/nbIP : 0;
	}
	
	Event lastBegin = null;
	Event lastEnd = null;
	IEventVisitor visitor = new EventVisitorAdapter() {

		@Override	public void visit(EnergyFailureEvent   e) { end(e); }
		@Override	public void visit(JobExecutionEndEvent e) {	end(e);	}
		@Override	public void visit(DeadlineMissedEvent  e) {	end(e);	}
		@Override	public void visit(CheckEffectiveDeadlineEvent e) {  }
		@Override
		public void visit(JobExecutionBeginEvent e) {
			long ipBegin = lastEnd != null ? lastEnd.getDate() : 0;
			long length = (e.getDate()>parameters.getSimulationEnd() ? parameters.getSimulationEnd() : e.getDate()) - ipBegin;
			if(length>0){
				sumIP += length;
				nbIP++;
			}
			ipBeginDetected = false;
			lastBegin = e;
		}

		private void end(Event e){
			ipBeginDetected = true;
			lastEnd = e;
		}

	};
	
	
	@Override
	public Object getAggregatedValue() {
		return aggregation.getPercent(aggregation.getAverage(), parameters.getSimulationEnd());
	}
}
