package yartiss.engine.simulation.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import yartiss.engine.simulation.IStatisticCriterion;

public class StatisticsManager {
	private static final Map<String, IStatisticCriterion> statisticsCriteria;
	private static final Map<String, Boolean> activatedCriteria;

	private static List<IStatisticCriterion> statisticsCriteriaList = new ArrayList<IStatisticCriterion>();
	
	static {
		statisticsCriteriaList.add(new FailureCountStatisticCriterion());
		statisticsCriteriaList.add(new SimulationDurationCriterion());
		statisticsCriteriaList.add(new SuccessStateStatisticsCriterion());
		statisticsCriteriaList.add(new AverageBusyPeriodStatisticCretirion());
		statisticsCriteriaList.add(new AverageIdlePeriodStatisticCretirion());
		statisticsCriteriaList.add(new AverageEnergyLevelStatisticCriterion());
		statisticsCriteriaList.add(new TimeAtEmaxStatisticCriterion());
		statisticsCriteriaList.add(new PreemptionsCountStatisticCriterion());
		statisticsCriteriaList.add(new AverageOverheadStatisticsCriterion());
		statisticsCriteriaList.add(new TimeAtEminStatisticCriterion());
		
		statisticsCriteria = new HashMap<String, IStatisticCriterion>();
		activatedCriteria = new HashMap<String, Boolean>();
		
		for (IStatisticCriterion criterion : statisticsCriteriaList) {
			statisticsCriteria.put(criterion.getName(), criterion);
			activatedCriteria.put(criterion.getName(), Boolean.TRUE);
		}
		setCriterionActivated(new SimulationDurationCriterion().getName(), false);
//		setCriterionActivated(new AverageBusyPeriodStatisticCretirion().getName(), false);
//		setCriterionActivated(new AverageIdlePeriodStatisticCretirion().getName(), false);
		setCriterionActivated(new TimeAtEmaxStatisticCriterion().getName(), false);
		setCriterionActivated(new TimeAtEminStatisticCriterion().getName(), false);
//		setCriterionActivated(new PreemptionsCountStatisticCriterion().getName(), false);
//		setCriterionActivated(new AverageOverheadStatisticsCriterion().getName(), false);
//		setCriterionActivated(new AverageEnergyLevelStatisticCriterion().getName(), false);
	}
	
	public static void registerStatisticsCriterion(IStatisticCriterion criterion){
		statisticsCriteria.put(criterion.getName(), criterion);
	}
	
	public static void setCriterionActivated(String criterion, boolean activated){
		activatedCriteria.put(criterion, Boolean.valueOf(activated));
	}
	
	public static IStatisticCriterion getInstance(String criterionName) {
		if(statisticsCriteria.containsKey(criterionName)){
			return statisticsCriteria.get(criterionName).newInstance();
		}
		throw new IllegalArgumentException(criterionName);
	}

	public static String[] getAvailableStatisticsCriterionNames() {
		String [] availible = new String [statisticsCriteria.size()];
		int i=0;
		for (IStatisticCriterion criterion : statisticsCriteriaList) {
			availible[i++] = criterion.getName();
		}
		return availible;
	}
	
	public static String[] getActivatedCriteria() {
		int count = 0;
		for (Boolean a : activatedCriteria.values())
			if(a)count ++;
		
		String [] availible = new String [count];
		int i=0;
		for (IStatisticCriterion criterion : statisticsCriteriaList) {
			if(activatedCriteria.get(criterion.getName()))
				availible[i++] = criterion.getName();
		}
		
		return availible;
	}
	
	public static boolean isActivated(String criterion){
		if(activatedCriteria.containsKey(criterion))
			return activatedCriteria.get(criterion);
		return false;
	}
}
