package yartiss.engine.simulation.system.taskset;

import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;

import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;

public class DynamicLaxityMonotonicTaskSet extends AbstractTaskSet{

	public DynamicLaxityMonotonicTaskSet() {
		super(new Comparator<ITask>() {
			@Override
			public int compare(ITask t1, ITask t2) {
				long laxity1  = t1.getDeadline() - t1.getRemainingCost();
				long laxity2 = t2.getDeadline() - t2.getRemainingCost();
				int cmp =  (int) (laxity1 - laxity2);
				if(cmp==0)
					return (int) (t1.getPriority() - t2.getPriority());
				return cmp;
			}
		});
	}

	@Override
	public SortedSet<ITask> getActiveTasks(long date) {
		SortedSet<ITask> activeTasks = new TreeSet<ITask>(comparator);
		for (ITask t : this)
			if (t.isActive())
				activeTasks.add(t);
		return activeTasks;
	}

	@Override
	public ITaskSet newInstance() {
		return new DynamicLaxityMonotonicTaskSet();
	}

	@Override
	public String getName() {
		return "Dynamic Laxity Monotonic";
	}

}
