package yartiss.engine.simulation.system.taskset;

import java.util.HashMap;
import java.util.Map;

import yartiss.engine.simulation.system.ITaskSet;

public class TaskSetFactory {

	private static final Map<String, ITaskSet> sets =  new HashMap<String, ITaskSet>();

	static {
		
		registerPolicy(new FixedPriorityTaskSet());
		registerPolicy(new DynamicDeadlineMonotonicTaskSet());
		registerPolicy(new LaxityGraphTaskSet());

		registerPolicy(new FixedPriorityTaskSet(), "priority");
		registerPolicy(new DynamicDeadlineMonotonicTaskSet(), "deadline");
		registerPolicy(new LaxityGraphTaskSet(), "laxity");
		
		registerPolicy(new DynamicLaxityMonotonicTaskSet());
		registerPolicy(new StaticDeadlineMonotonicTaskSet());
		registerPolicy(new StaticRateMonotonicTaskset());
		registerPolicy(new StaticEnergyCostMonotonicTaskSet());
		registerPolicy(new StaticEnergyDeadlineMonotonicTaskSet());
		registerPolicy(new StaticInstantaneousEnergyConsRateTaskSet());

	}

	public static void registerPolicy(ITaskSet set){
		sets.put(set.getName(), set);
	}

	public static void registerPolicy(ITaskSet set, String name){
		sets.put(name, set);
	}

	public static ITaskSet getNewInstance(String taskSetTypeName) {
		if(sets.containsKey(taskSetTypeName))
			return sets.get(taskSetTypeName).newInstance();
		throw new IllegalArgumentException(taskSetTypeName);
	}
	
	public static String[] getAvailableTaskSets() {
		String [] availible = new String [sets.size()];
		int i=0;
		for (String type : sets.keySet()) {
			availible[i++] = type;
		}
		return availible;
	}
}
