package yartiss.engine.simulation.system.taskset.generation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;

import yartiss.engine.simulation.io.many.ITasksetsWriter;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.SystemMetrics;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;

public class LimitedHyperperiodGenerator implements ITaskSetGenerator{

	private long [][] M = buildMatrix();
	private double u1, u2, d1, d2, o1, o2, U,Ue;
	private int n;
	private Random random = new Random(System.nanoTime());
	
	
	
	public LimitedHyperperiodGenerator(double u1, double u2, double d1,
			double d2, double o1, double o2, double procLoad, double energyLoad, int nbTasks) {
		super();
		this.u1 = u1;
		this.u2 = u2;
		this.d1 = d1;
		this.d2 = d2;
		this.o1 = o1;
		this.o2 = o2;
		U = procLoad;
		Ue = energyLoad;
		this.n = nbTasks;
	}

	@Override
	public String getName() {
		return "Limited Hyper Period Generator";
	}

	@Override
	public String getDescription() {
		// TODO Auto-generated method stub
		return "";
	}

	@Override
	public ITask[] generate() {
		ArrayList<ITask> tasks = new ArrayList<ITask>();
		int i=0;
		double currentU = 0;
		long Ti,Ci,Di,Oi,Ei;
		
		while(currentU < U && i<n){
			
			Ti = generatePeriod(M);
			Ci = Math.max(1, Math.round(rand(u1,u2)*Ti));
			Oi = Math.round(rand(o1,o2)*Ti);
			Di = Math.round((Ti-Ci)*rand(d1,d2))+Ci;
			Ei=Ci;
			double ct = ((double)Ci)/Ti;
			if(currentU + ct <=1 ){
				currentU += ct;
				tasks.add(SchedulableFactory.newInstance("simple", Oi, Ci, Ei,Ti, Di, i+1));
			}
			i++;
		}
		System.out.println("n="+i);
		ITask [] set = new ITask [tasks.size()];
		return tasks.toArray(set);
	}
	
	
	private double rand(double a, double b){
		return a + random.nextDouble() * (b-a);
	}
	
	private long [][] buildMatrix(){
		long [][] matrix = new long [][]{
				{1,  2,  2,  4,  4,  4,  8, 16, 16},
				{1,  3,  3, 3,  9,  9,  9},
				{1,  5,  5,  5, 25, 25},
				{1,  1,  1,  7,  7},
		};
		return matrix;
	}
	
	
	private long generatePeriod(long [][] matrix){
		long period = 1;
		
		for (long[] line : matrix) {
			int p = random.nextInt(line.length);
			period *= line[p];
		}
		return period;
	}
	
	public static void mainu(String[] args) throws Exception {
		int s = 5, n=19 ,nb=10;
		int count =  (n-s)*(n-s)*nb;
		int pr = 15;
		int nbTasks = 8;
		int counter = 0;
		final ITasksetsWriter writer = TasksetsIOFactory.newTasksetsWriterInstance("xml","file.xml",pr);

		for (int i = s; i < n; i++) {
			for (int j = s; j < n; j++) {
				for (int k = 0; k < nb; k++) {
					double u  =  Math.round((0.05*i) * 1000) / 1000.0d;
					double ue =  Math.round((0.05*j) * 1000) / 1000.0d;
					System.out.println("---- u="+u+"	ue="+ue);
					LimitedHyperperiodGenerator gen = new LimitedHyperperiodGenerator(0.3,0.9, 1, 1, 0, 0, u, ue, nbTasks);
					ITask [] taskset = gen.generate();
					writer.write(taskset);
					
					System.out.println();

					u  = Math.round(SystemMetrics.utilisation(Arrays.asList(taskset)) * 1000) / 1000.0d;
					ue = Math.round(SystemMetrics.energyUtilization(Arrays.asList(taskset), pr) * 1000) / 1000.0d;
					System.out.println("++++ u="+u+"	ue="+ue+"  hp="+SystemMetrics.hyperPeriod(Arrays.asList(taskset)));
					counter++;
					System.out.println("---------------------------------------"+counter+"/"+count);
					
				}
			}
		}
		writer.close();
	}

}
