package yartiss.engine.test.simulation.io;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.DefaultEnergyProfile;
import yartiss.engine.simulation.io.one.TextSimulationParametersReader;
import yartiss.engine.simulation.io.one.TextSimulationParametersWriter;
import yartiss.engine.simulation.io.one.XmlSimulationParametersReader;
import yartiss.engine.simulation.io.one.XmlSimulationParametersWriter;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SimpleTask;

public class SimulationParametersIOTest {
	
	String file = "testfile";
	
	@Before
	public void setUp(){
		
	}
	
	@Test
	public void TextIOTest() throws FileNotFoundException, IOException {
		SimulationParameters expected = buildParams();
		new TextSimulationParametersWriter().write(expected, file);
		SimulationParameters result = new TextSimulationParametersReader().read(file);
		assertParams(expected, result);
	}
	
	@Test
	public void XmlIOTest() throws FileNotFoundException, IOException {
		
		SimulationParameters expected = buildParams();
		new XmlSimulationParametersWriter().write(expected, file);
		SimulationParameters result = new XmlSimulationParametersReader().read(file);
		assertParams(expected, result);
	}
	
	@After
	public void after(){
		new File(file).delete();
	}
	
	private void assertParams(SimulationParameters expected, SimulationParameters result){
		Assert.assertNotNull(result);
		Assert.assertNotNull(result.getEnergyProfile());
		Assert.assertNotNull(result.getSchedulingPolicy());
		Assert.assertNotNull(result.getTaskSet());
		
		
		Assert.assertEquals(expected.getNbProc()	        , result.getNbProc());
		Assert.assertEquals(expected.getSimulationEnd()	    , result.getSimulationEnd());
		
		//Assert.assertEquals(expected.isEnergyAware()	    , result.isEnergyAware());
		//Assert.assertEquals(expected.isHardRealtime()	    , result.isHardRealtime()));
		
		Assert.assertEquals(expected.getSchedulingPolicy().getPolicyName()  , result.getSchedulingPolicy().getPolicyName());
		
		for (int i = 0; i < expected.getSchedulingPolicy().getParameters().size(); i++) {
			Assert.assertEquals(expected.getSchedulingPolicy().getParameters().get(i), result.getSchedulingPolicy().getParameters().get(i));
		}
		
		Assert.assertEquals(expected.getTaskSet().getClass().getName()	    , result.getTaskSet().getClass().getName());
		
		List<ITask> resultsTasks = new ArrayList<ITask>();
		List<ITask> expectedTasks = new ArrayList<ITask>();
		for (ITask task : result.getTaskSet()) 	resultsTasks.add(task);
		for (ITask task : expected.getTaskSet()) expectedTasks.add(task);
		
		for (int i = 0; i < expectedTasks.size(); i++) {
			assetTask(expectedTasks.get(i), resultsTasks.get(i));
		}
		
		Assert.assertEquals(expected.getEnergyProfile().getClass().getName(), result.getEnergyProfile().getClass().getName());
		Assert.assertEquals(expected.getEnergyProfile().getMaxEnergyLevel() , result.getEnergyProfile().getMaxEnergyLevel());
		Assert.assertEquals(expected.getEnergyProfile().getMinEnergyLevel() , result.getEnergyProfile().getMinEnergyLevel());
		Assert.assertEquals(expected.getEnergyProfile().getPowerRate()      , result.getEnergyProfile().getPowerRate());
	}

	public static void assetTask(ITask expected, ITask result) {
		Assert.assertEquals(expected.getType(), result.getType());
		Assert.assertEquals(expected.getFirstRelease(), result.getFirstRelease());
		Assert.assertEquals(expected.getWcet(), result.getWcet());
		Assert.assertEquals(expected.getWcee(), result.getWcee());
		Assert.assertEquals(expected.getPeriod(), result.getPeriod());
		Assert.assertEquals(expected.getDeadline(), result.getDeadline());
		Assert.assertEquals(expected.getPriority(), result.getPriority());
		
	}
	
	
	private SimulationParameters buildParams(){
		ISchedulingPolicy policy = SchedulingPolicyFactory.getInstance("EH_PFP_5");
		ArrayList<Integer> parameters = new ArrayList<Integer>();
		parameters.add(5);
		parameters.add(7);
		policy.setParameters(parameters);
		IEnergyProfile ep = new DefaultEnergyProfile(15, 200, 0, 50);
		ITaskSet set = policy.createTaskSet();
		for (int i = 1; i < 5; i++) {
			set.addTask(new SimpleTask(i, i*5, i*10, i*7, i*7, i+1, null));
		}
		return new SimulationParameters(policy, 1000, ep, set, 2, false, false);
	}
	
}
