package yartiss.engine.test.simulation.io;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import yartiss.engine.simulation.io.many.ITasksetsReader;
import yartiss.engine.simulation.io.many.ITasksetsWriter;
import yartiss.engine.simulation.io.many.TextTasksetsReader;
import yartiss.engine.simulation.io.many.TextTasksetsWriter;
import yartiss.engine.simulation.io.many.XmlTasksetsReader;
import yartiss.engine.simulation.io.many.XmlTasksetsWriter;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.taskset.generation.LimitedHyperperiodGenerator;

public class TasksetsIOTest {

	String file = "testfile";
	long pr = 15;
	@Test
	public void XmlIOTest() throws FileNotFoundException, IOException {
		
		List<ITask[]> expected = buildExpected();
		ITasksetsWriter writer = new TextTasksetsWriter(file, pr);
		
		for (ITask[] set : expected) {
			writer.write(set);
		}
		writer.close();
		
		ITasksetsReader reader = new TextTasksetsReader(file);
		
		List<ITask[]> result = new ArrayList<ITask[]>();
		ITask [] set = null;
		while( (set = reader.nextTaskset())!=null){
			result.add(set);
		}
		
		for (int i = 0; i < expected.size(); i++) {
			assertTaskset(expected.get(i), result.get(i));
		}
	}
	
	private void assertTaskset(ITask[] expected, ITask[] result) {
		Assert.assertEquals(expected.length, result.length);
		for (int i = 0; i < expected.length; i++) {
			SimulationParametersIOTest.assetTask(expected[i], result[i]);
		}
	}

	@Test
	public void TextIOTest() throws Exception {
		List<ITask[]> expected = buildExpected();
		ITasksetsWriter writer = new XmlTasksetsWriter(file,pr);
		
		for (ITask[] set : expected) {
			writer.write(set);
		}
		writer.close();
		
		ITasksetsReader reader = new XmlTasksetsReader(file);
		
		List<ITask[]> result = new ArrayList<ITask[]>();
		ITask [] set = null;
		while( (set = reader.nextTaskset())!=null){
			result.add(set);
		}
		
		for (int i = 0; i < expected.size(); i++) {
			assertTaskset(expected.get(i), result.get(i));
		}
	}
	
	
	
	private List<ITask[]> buildExpected(){
		
		List<ITask[]> list = new ArrayList<ITask[]>();
		
		int s = 5, n=19 ,nb=1;
		int nbTasks = 4;
		
		for (int i = s; i < n; i++) {
			for (int j = s; j < n; j++) {
				for (int k = 0; k < nb; k++) {
					double u  =  Math.round((0.05*i) * 1000) / 1000.0d;
					double ue =  Math.round((0.05*j) * 1000) / 1000.0d;
					LimitedHyperperiodGenerator gen = new LimitedHyperperiodGenerator(0.3,0.9, 1, 1, 0, 0, u, ue, nbTasks);
					ITask [] taskset = gen.generate();
					list.add(taskset);
				}
			}
		}
		return list;
	}
	
}
