package yartiss.engine.test.simulation.scheduling.policies;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.IEventAddedListener;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.system.ITaskSet;

public abstract class AbstractPolicyTest {

	protected ISchedulingPolicy policy;
	protected IEnergyProfile energyProfile = EnergyProfileFactory.getInstance("default",Integer.MAX_VALUE, Integer.MAX_VALUE, 0, Integer.MAX_VALUE );;
	private static List<String> acceptedEvents;
	
	
	
	static {
		acceptedEvents = new ArrayList<String>();
		acceptedEvents.add("activate_task" );
		acceptedEvents.add("job_execution_end");
		acceptedEvents.add("job_execution_begin");
		acceptedEvents.add("deadline_missed");
	}
	
	
	@Test
	public void policyTest(){
		final List<Event> generatedEvents = new ArrayList<Event>();
		ITaskSet taskset = policy.createTaskSet();
		generateTasks(taskset);
		energyProfile.reset();
		
		Simulation sim = new Simulation(new SimulationParameters(policy, getsimulationDuration(), energyProfile, taskset,getNbPtoc(), isEnergyAware(), isHardRealtime()));
		sim.addEventAddedListerner(new IEventAddedListener() {
			@Override
			public void perfomredAction(Event e) {
				if(acceptedEvents.contains(e.toString())){
					generatedEvents.add(e);
				}
			}
		});
		
		sim.simulate( );
		
		final Event [] expectedEvents = getExpectdEvents(taskset);
		assertEvents(expectedEvents, generatedEvents);
	}

	private void assertEvents(Event [] expectedEvents, List<Event> generatedEvents) {
		System.out.println("-------------------------------------------------");
		System.out.println("date\t: event name\t\t: E\t: state");
		System.out.println("-------------------------------------------------");
		
		for (Event event : expectedEvents) {
			System.out.print(event.getDate()+"\t: t"+event.getTask().getPriority()+"\t: "+event.toString()+"\t: "+event.getMetric(Event.EnergyLevel)+"\t:");
			Assert.assertTrue(generatedEvents.contains(event));
			System.out.println(" OK");
		}
	}
	
	protected Map<String,Long> metrics(long ... values ){
		Map<String, Long> m = new HashMap<String, Long>();
		m.put(Event.EnergyLevel, values[0]);
		return m;
	}
	
	protected int getsimulationDuration() {
		return 100;
	}
	
	protected int getNbPtoc() {
		return 5;
	}
	
	protected boolean isCheckEnergy(){
		return true;
	}

	@Before
	public abstract void setUp();
	protected abstract void generateTasks(ITaskSet ts) ;
	protected abstract Event[] getExpectdEvents(ITaskSet taskset);
	protected abstract boolean isEnergyAware();
	protected abstract boolean isHardRealtime();

}
