package yartiss.engine.test.simulation.scheduling.policies.dp;

import org.junit.Before;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.energy.profiles.DefaultEnergyProfile;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.dp.EDFMultiProc;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.test.simulation.scheduling.policies.AbstractPolicyTest;

public class EDFMultiProcSchedulingPolicyTest extends AbstractPolicyTest{
	private int nbProc = 2;

	private ITask [] tasks = new ITask[]{
			 SchedulableFactory.newInstance("simple",0, 1, 1, 2, 2, 1),
			 SchedulableFactory.newInstance("simple",0, 1, 1, 4, 4, 2),
			 SchedulableFactory.newInstance("simple",0, 1, 1, 8, 8, 3)
	};
	
	@Before
	@Override
	public void setUp(){
		policy = new EDFMultiProc();
		energyProfile = new DefaultEnergyProfile(5, 100, 1, 100);
	}


	@Override
	protected Event[] getExpectdEvents(ITaskSet taskset) {
		switch (nbProc) {
		case 2: return getExpectdEvents2(taskset);
		}
		return getExpectdEvents2(taskset);
	}
	
	private Event[] getExpectdEvents2(ITaskSet taskset) {
		EventGenerator evGen = new EventGenerator();
		Event [] events = new Event []{
				
			evGen.getNewInstance("activate_task",tasks[0], 		0, null), 
			evGen.getNewInstance("activate_task",tasks[1], 		0, null), 
			evGen.getNewInstance("activate_task",tasks[2], 		0, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],0, null), 
			evGen.getNewInstance("job_execution_begin",tasks[1],0, null), 
			
			evGen.getNewInstance("job_execution_end",tasks[0], 	1, null), 
			evGen.getNewInstance("job_execution_end",tasks[1], 	1, null), 
			evGen.getNewInstance("job_execution_begin",tasks[2],1, null), 
			
			evGen.getNewInstance("activate_task",tasks[0], 		2, null), 
			evGen.getNewInstance("job_execution_end",tasks[2], 	2, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],2, null), 
			
			evGen.getNewInstance("job_execution_end",tasks[0], 	3, null), 
			
			evGen.getNewInstance("activate_task",tasks[0], 		4, null), 
			evGen.getNewInstance("activate_task",tasks[1], 		4, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],4, null), 
			evGen.getNewInstance("job_execution_begin",tasks[1],4, null), 
			
			evGen.getNewInstance("job_execution_end",tasks[0], 	5, null), 
			evGen.getNewInstance("job_execution_end",tasks[1], 5, null), 
			
			evGen.getNewInstance("activate_task",tasks[0], 		6, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],6, null), 
			
			evGen.getNewInstance("job_execution_end",tasks[0], 	7, null), 
			
			evGen.getNewInstance("activate_task",tasks[0], 		8, null), 
			evGen.getNewInstance("activate_task",tasks[1], 		8, null), 
			evGen.getNewInstance("activate_task",tasks[2], 		8, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],8, null), 
			evGen.getNewInstance("job_execution_begin",tasks[1],8, null), 
			
			evGen.getNewInstance("job_execution_end",tasks[0], 	9, null), 
			evGen.getNewInstance("job_execution_end",tasks[1], 	9, null), 
			evGen.getNewInstance("job_execution_begin",tasks[2],9, null),
			 
			evGen.getNewInstance("activate_task",tasks[0], 		10, null), 
			evGen.getNewInstance("job_execution_end",tasks[2], 	10, null), 
			evGen.getNewInstance("job_execution_begin",tasks[0],10, null), 
		};
		for (Event event : events) {
			event.setMetric(Event.EnergyLevel, 100);
		}
		return events;
	}

	@Override
	protected void generateTasks(ITaskSet taskset) {
		for (ITask task : tasks) {
			taskset.addTask(task);
		}
	}

	@Override
	protected int getsimulationDuration() {
		return 11;
	}


	@Override
	protected int getNbPtoc() {
		return nbProc;
	}
	@Override
	protected boolean isCheckEnergy(){
		return false;
	}


	@Override
	protected boolean isEnergyAware() {
		return false;
	}


	@Override
	protected boolean isHardRealtime() {
		return true;
	}
	
}
