package yartiss.engine.test.simulation.scheduling.policies.dp;

import org.junit.Before;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.energy.profiles.DefaultEnergyProfile;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.dp.EDF;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.test.simulation.scheduling.policies.AbstractPolicyTest;

public class EDFSchedulingPolicyTest extends AbstractPolicyTest{
	
	private int nbProc = 1;

	private ITask [] tasks = new ITask[]{
			 SchedulableFactory.newInstance("simple",0, 2, 2,  5,  5, 1),
			 SchedulableFactory.newInstance("simple",0, 1, 1,  8,  8, 2),
			 SchedulableFactory.newInstance("simple",0, 4, 4, 10, 10, 3)
	};
	
	@Before
	@Override
	public void setUp(){
		policy = new EDF();
		energyProfile = new DefaultEnergyProfile(1, 100, 1, 100);
	}


	@Override
	protected Event[] getExpectdEvents(ITaskSet taskset) {
		EventGenerator evGen = new EventGenerator();
		Event [] events = new Event []{
				evGen.getNewInstance("activate_task"			, tasks[0], 0, null), 
				evGen.getNewInstance("activate_task"			, tasks[1], 0, null), 
				evGen.getNewInstance("activate_task"			, tasks[2], 0, null), 
				
				evGen.getNewInstance("job_execution_begin"		, tasks[0], 0, null), 
				evGen.getNewInstance("job_execution_end"		, tasks[0], 2, null), 

				evGen.getNewInstance("job_execution_begin"		, tasks[1], 2, null), 
				evGen.getNewInstance("job_execution_end"		, tasks[1], 3, null), 

				evGen.getNewInstance("job_execution_begin"		, tasks[2], 3, null), 
				evGen.getNewInstance("job_execution_end"		, tasks[2], 5, null), 

				evGen.getNewInstance("activate_task"			, tasks[0], 5, null), 
				
				evGen.getNewInstance("job_execution_begin"		, tasks[0], 5, null), 
				evGen.getNewInstance("job_execution_end"		, tasks[0], 7, null), 

				evGen.getNewInstance("job_execution_begin"		, tasks[2], 7, null), 
				evGen.getNewInstance("activate_task"			, tasks[1], 8, null),  
				evGen.getNewInstance("job_execution_end"		, tasks[2], 9, null), 

				evGen.getNewInstance("job_execution_begin"		, tasks[1], 9, null), 
				evGen.getNewInstance("job_execution_end"		, tasks[1], 10, null), 

				evGen.getNewInstance("activate_task"			, tasks[0], 10, null), 
				evGen.getNewInstance("activate_task"			, tasks[2], 10, null), 
				
		};
		for (Event event : events) {
			event.setMetric(Event.EnergyLevel, 100);
		}
		return events;
	}


	@Override
	protected void generateTasks(ITaskSet taskset) {
		for (ITask task : tasks) {
			taskset.addTask(task);
		}
	}


	@Override
	protected int getsimulationDuration() {
		return 11;
	}


	@Override
	protected int getNbPtoc() {
		return nbProc;
	}
	
	@Override
	protected boolean isEnergyAware() {
		return false;
	}


	@Override
	protected boolean isHardRealtime() {
		return true;
	}

}
