package yartiss.engine.test.simulation.scheduling.policies.fp;

import java.util.ArrayList;
import java.util.List;

import org.junit.Before;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.energy.profiles.DefaultEnergyProfile;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.fp.PFPASAP;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.test.simulation.scheduling.policies.AbstractPolicyTest;

public class EHPFP1SchedulingPolicyTest extends AbstractPolicyTest{
	
	private ITask [] tasks = new ITask[]{
			
			 SchedulableFactory.newInstance("simple", 0, 1, 2,  8,  8, 1),
			 SchedulableFactory.newInstance("simple", 0, 2, 4,  5,  5, 2),
			 SchedulableFactory.newInstance("simple", 0, 4, 8, 12, 12, 3)
	};
	
	@Before
	@Override
	public void setUp(){
		policy = new PFPASAP();
		List<Integer> params = new ArrayList<Integer>();
		params.add(2);
		policy.setParameters(params);
		energyProfile = new DefaultEnergyProfile(1, 10, 1, 5);
	}
	
	
	@Override
	protected Event[] getExpectdEvents(ITaskSet taskset) {
		EventGenerator EventGenerator = new EventGenerator();
		Event [] events = new Event []{
				EventGenerator.getNewInstance("activate_task"			, tasks[0], 0, metrics(5)), 
				EventGenerator.getNewInstance("activate_task"			, tasks[1], 0, metrics(5)), 
				EventGenerator.getNewInstance("activate_task"			, tasks[2], 0, metrics(5)), 
				
				EventGenerator.getNewInstance("job_execution_begin"		, tasks[0], 0, metrics(5)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[0], 1, metrics(4)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[1], 1, metrics(4)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[1], 3, metrics(2)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[2], 3, metrics(2)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[2], 4, metrics(1)), 

				EventGenerator.getNewInstance("activate_task"			, tasks[1], 5, metrics(2)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[1], 6, metrics(3)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[1], 8, metrics(1)),
				
				EventGenerator.getNewInstance("activate_task"			, tasks[0], 8, metrics(1)),  
				EventGenerator.getNewInstance("activate_task"			, tasks[1], 10, metrics(3)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[0], 10, metrics(3)),
				EventGenerator.getNewInstance("job_execution_end"		, tasks[0], 11, metrics(2)),

				
				EventGenerator.getNewInstance("job_execution_begin"		, tasks[1], 11, metrics(2)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[1], 12, metrics(1)), 
				
				EventGenerator.getNewInstance("activate_task"			, tasks[2], 12, metrics(1)), 
				EventGenerator.getNewInstance("deadline_missed"			, tasks[2], 12, metrics(1)), 
		};
		return events;
	}
	
	


	@Override
	protected void generateTasks(ITaskSet taskset) {
		for (ITask task : tasks) {
			taskset.addTask(task);
		}
	}


	@Override
	protected int getsimulationDuration() {
		return 13;
	}
	
	@Override
	protected int getNbPtoc() {
		return 1;
	}
	
	@Override
	protected boolean isEnergyAware() {
		return true;
	}
	
	@Override
	protected boolean isHardRealtime() {
		return true;
	}
}
