package yartiss.engine.test.simulation.scheduling.policies.fp;

import java.util.ArrayList;
import java.util.List;
import org.junit.Before;

import yartiss.engine.simulation.Event;
import yartiss.engine.simulation.energy.profiles.DefaultEnergyProfile;
import yartiss.engine.simulation.events.EventGenerator;
import yartiss.engine.simulation.scheduling.policies.fp.PFPSlackTime;
import yartiss.engine.simulation.scheduling.policies.tools.FPSlackTimeManager;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.test.simulation.scheduling.policies.AbstractPolicyTest;

public class EHPFP4SchedulingPolicyTest extends AbstractPolicyTest {

	private ITask [] tasks = new ITask[]{
			 SchedulableFactory.newInstance("simple", 0, 1, 2,  10,  10, 1),
			 SchedulableFactory.newInstance("simple", 0, 2, 4,  10,  10, 2),
			 SchedulableFactory.newInstance("simple", 0, 4, 8, 12, 12, 3)
	};
	
	@Before
	@Override
	public void setUp(){
		policy = new PFPSlackTime(new FPSlackTimeManager());
		List<Integer> params = new ArrayList<Integer>();
		policy.setParameters(params);
		energyProfile = new DefaultEnergyProfile(1, 3, 1, 2);
	}
	
	
	@Override
	protected Event[] getExpectdEvents(ITaskSet taskset) {
		EventGenerator EventGenerator = new EventGenerator();
		Event [] events = new Event []{
				EventGenerator.getNewInstance("activate_task"			, tasks[0], 0, metrics(2)), 
				EventGenerator.getNewInstance("activate_task"			, tasks[1], 0, metrics(2)), 
				EventGenerator.getNewInstance("activate_task"			, tasks[2], 0, metrics(2)), 
				
				EventGenerator.getNewInstance("job_execution_begin"		, tasks[0], 0, metrics(2)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[0], 1, metrics(1)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[1], 3, metrics(3)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[1], 5, metrics(1)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[2], 6, metrics(2)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[2], 7, metrics(1)), 

				EventGenerator.getNewInstance("job_execution_begin"		, tasks[2], 8, metrics(2)), 
				EventGenerator.getNewInstance("job_execution_end"		, tasks[2], 9, metrics(1)), 
				
				EventGenerator.getNewInstance("activate_task"			, tasks[1], 10, metrics(2)), 
				EventGenerator.getNewInstance("activate_task"			, tasks[0], 10, metrics(2)), 
				
				EventGenerator.getNewInstance("job_execution_begin"		, tasks[0], 10, metrics(2)),
				EventGenerator.getNewInstance("job_execution_end"		, tasks[0], 11, metrics(1)),
				
				EventGenerator.getNewInstance("activate_task"			, tasks[2], 12, metrics(2)), 

				EventGenerator.getNewInstance("deadline_missed"			, tasks[2], 12, metrics(2)), 
		}; 
		return events;
	}

	@Override
	protected void generateTasks(ITaskSet taskset) {
		for (ITask task : tasks) {
			taskset.addTask(task);
		}
	}

	@Override
	protected int getsimulationDuration() {
		return 13;
	}
	
	
	@Override
	protected boolean isEnergyAware() {
		return true;
	}
	
	@Override
	protected boolean isHardRealtime() {
		return true;
	}
}
