package yartiss.engine.test.simulation.scheduling.policies.fp;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.engine.simulation.system.SystemMetrics;
import yartiss.engine.simulation.system.taskset.generation.FileTaskSetGenerator;
import yartiss.engine.simulation.system.taskset.generation.ITaskSetGenerator;
import yartiss.engine.simulation.system.taskset.generation.UUniFastDiscardTaskSetGen;
import yartiss.engine.simulation.system.taskset.generation.task.DefaultTaskGenerator;

public class FPegSchedulingPolicyTest {

	private static final int end = 10000;
	
	@Test
	public void Test1() {
		
	}

	// @Test
	public void Tset1() {
		int nbEntries = 30;
		int nbTasks = 6, nbProc = 1;
		double Ue = 0.8, U = 0.8;

		IEnergyProfile energyProfile = EnergyProfileFactory.getInstance("defaultEnergyProfile", 20, 200, 0);
		ISchedulingPolicy[] policies = buildSchedulingPolicies("FPeg",	"EH_PFP_4");
		boolean[][] faisibility = new boolean[30][2];
		double[] Ues = new double[30];
		double[] Us = new double[30];
		for (int i = 0; i < policies.length; i++) {

			ITaskSetGenerator taskSetGenerator = new UUniFastDiscardTaskSetGen(
					new DefaultTaskGenerator(), nbTasks, nbProc, U, Ue,
					energyProfile.getPowerRate(),
					energyProfile.getMaxEnergyLevel(), 0, false, true);
			// ITaskSetGenerator taskSetGenerator = new
			// FileTaskSetGenerator(getClass().getResourceAsStream("/resources/tasks_sets"),
			// getClass().getResourceAsStream("/resources/energy_"+energyProfile.getMaxEnergyLevel()+"_"+energyProfile.getPowerRate()+".txt"),
			// energyProfile.getPowerRate());

			String pol = policies[i].getPolicyName();
			System.out.println("-----------" + pol + " :  started");
			for (int j = 0; j < nbEntries; j++) {

				policies = buildSchedulingPolicies("FPeg", "EH_PFP_4");
				energyProfile.reset();
				ITask[] tasks = taskSetGenerator.generate();
				for (ITask task : tasks) {
					System.out.println(task);
				}
				System.out.println("generationn finished");
				Simulation sim = new Simulation(buildSimulationParameters(
						energyProfile, policies[i], tasks, end));
				sim.setLoggingEnabled(false);
				// if(j==15)
				sim.simulate();
				faisibility[j][i] = sim.getStatistics().isSuccess();
				Ues[j] = SystemMetrics.energyUtilization(sim
						.getSimulationParameters().getTaskSet(), energyProfile
						.getPowerRate());
				Us[j] = SystemMetrics.utilisation(sim.getSimulationParameters()
						.getTaskSet());
//				System.out.println("system " + j + " ok "
//						+ sim.getStatistics().getSimulationDuration());
			}
			System.out.println("-----------" + pol + " :  finish");
		}

		for (int i = 0; i < faisibility.length; i++) {
			System.out.print(i + "\t: ");
			for (int j = 0; j < faisibility[i].length; j++) {
				System.out.print(faisibility[i][j] + "\t");
			}
			System.out.println("Ue=" + Ues[i] + "\tU=" + Us[i]);
		}
	}

	// @Test
	public void Tset2() {
		IEnergyProfile energyProfile = EnergyProfileFactory.getInstance(
				"defaultEnergyProfile", 20, 200, 0);
		ITaskSetGenerator taskSetGenerator = new FileTaskSetGenerator(
				getClass().getResourceAsStream("/resources/systems/tasks_sets"),
				getClass().getResourceAsStream(
						"/resources/systems/energy_"
								+ energyProfile.getMaxEnergyLevel() + "_"
								+ energyProfile.getPowerRate() + ".txt"),
				energyProfile.getPowerRate());

		energyProfile = EnergyProfileFactory.getInstance(
				"defaultEnergyProfile", 20, 200, 0);
		for (int i = 0; i < 5; i++) {
			ISchedulingPolicy[] policies = buildSchedulingPolicies("FPeg",
					"EH_PFP_4");
			energyProfile.reset();
			taskSetGenerator = new FileTaskSetGenerator(getClass()
					.getResourceAsStream("/resources/tasks_sets"), getClass()
					.getResourceAsStream(
							"/resources/energy_"
									+ energyProfile.getMaxEnergyLevel() + "_"
									+ energyProfile.getPowerRate() + ".txt"),
					energyProfile.getPowerRate());
			ITask[] tasks = null;
			for (int j = 0; j < 5; j++) {
				tasks = taskSetGenerator.generate();
			}

			System.out.println("###########################################");
			Simulation sim = new Simulation(buildSimulationParameters(
					energyProfile, policies[0], tasks, end));
			sim.setLoggingEnabled(true);
			sim.setLogFile("results/sim_" + i + ".sim");
			sim.simulate();
//			System.out.println("system 2" + " ok "
//					+ sim.getStatistics().getSimulationDuration());
		}

	}

	private ISchedulingPolicy[] buildSchedulingPolicies(
			String... policiesParams) {
		ISchedulingPolicy[] policies = new ISchedulingPolicy[policiesParams.length];
		int i = 0;
		for (String policy : policiesParams) {
			String[] params = policy.split(" ");
			String policyName = params[0];
			policies[i] = SchedulingPolicyFactory.getInstance(policyName);
			List<Integer> paramss = new ArrayList<Integer>();
			for (int j = 1; j < params.length; j++) {
				paramss.add(Integer.parseInt(params[j]));
			}
			policies[i].setParameters(paramss);
			i++;
		}
		return policies;
	}

	private SimulationParameters buildSimulationParameters(
			IEnergyProfile energyProfile, ISchedulingPolicy policy,
			ITask[] tasks, int simulationDuration) {
		ITaskSet taskset = policy.createTaskSet();
		int nbTasks = tasks.length;
		for (int i = 0; i < nbTasks; i++) {
			taskset.addTask(tasks[i].cloneTask());
		}
		return new SimulationParameters(policy, simulationDuration,
				energyProfile, taskset, 1, true, true);
	}
	
	
}
