package yartiss.framework.concurrency;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BlockingQueue;

public class Consumer<E> extends  Thread{
	
	private final   BlockingQueue<QueueEntry<E>> queue;
	private final 	List<IElementConsumedListener<E>> listeners = new ArrayList<IElementConsumedListener<E>>();
	private final 	List<IElementConsumedListener<E>> finishListeners = new ArrayList<IElementConsumedListener<E>>();

	private static int id = 0;
	public Consumer(BlockingQueue<QueueEntry<E>> queue, IElementConsumedListener<E> onConsumedListener){
		this.queue = queue;
		addElementConsumedListener(onConsumedListener);
		id++;
	}
	
	@Override
	public void run(){
		E entry = null;
		while(true){
			try {
				entry = queue.take().getValue();
				id--;
				if(entry != null){
					fireConsumed(entry); 
				}
				else {
					//TODO
					fireFinished(null);
					break;
				}
			} 
			catch (InterruptedException e) {}
		}
		//System.out.println("consumer finished");
	}
	
	public void addElementConsumedListener(IElementConsumedListener<E> l){
		listeners.add(l);
	}
	
	public void addFinishListener(IElementConsumedListener<E> l){
		finishListeners.add(l);
		System.out.println("id="+id);
	}
	
	protected void fireFinished(E e){
		for (IElementConsumedListener<E> l : finishListeners) {
			l.consume(e);
		}
	}
	
	protected void fireConsumed(E e){
		for (IElementConsumedListener<E> l : listeners) {
			l.consume(e);
		}
	}
	
	public void shutdown(){
		 try {queue.put(new QueueEntry<E>(null));} 
		 catch (InterruptedException e) {}
	}
}
