package yartiss.framework.concurrency;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class MultiThreadProducer<E> extends AbstractProducer<E>{

	private final ExecutorService executor ;
	private final int maxThread = 500;
	private final BoundedExecutor bex;

	public MultiThreadProducer(BlockingQueue<QueueEntry<E>> queue) {
		super(queue);
		executor = Executors.newFixedThreadPool(maxThread);
		bex = new BoundedExecutor(executor, maxThread);
	}

	@Override
	public void produce(final Callable<E> callable){
		try {
			bex.submitTask(new Runnable() {

				@Override	public void run() {
					try {
						if(callable==null)
							queue.put(new QueueEntry<E>(null));

						E result = callable.call();

						if(result == null){
							//stop();
							return ;
						}
						queue.put(new QueueEntry<E>(result));
					} catch (Exception e) {	}
				}
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}


	@Override
	public void awaitTermination(){
		executor.shutdown();
		while(!executor.isTerminated()){
			try {
				System.out.println(executor.awaitTermination(1, TimeUnit.DAYS));
			} catch (InterruptedException e) {}
		}
		System.out.println("Producer : awaitTermination finished");
	}

	@Override
	public void stop(){
		executor.shutdownNow();
		System.out.println("Producer : stop finished");
	}
}
