package yartiss.service.model;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.xy.XYSeries;

import yartiss.engine.simulation.SimulationStatistics;

public class Metric {
	private final String name;
	private final Map<String, Integer> nbReceivedByPolicy = new HashMap<String, Integer>();
	private final Map<String, Map<String,SimulationStatistics>> policiesStatsPerScenario = new HashMap<String, Map<String, SimulationStatistics>>();
	private final DefaultCategoryDataset  data = new DefaultCategoryDataset (); 
	private final Map<String, XYSeries> curves = new HashMap<String, XYSeries>();

	public Metric(String name){
		this.name = name;
	}


	public void updatePolicyStats(Scenario scenario,String policyName, SimulationStatistics newStats){
		XYSeries series = curves.containsKey(policyName) ? curves.get(policyName) : new  XYSeries(policyName);
		Map<String,SimulationStatistics> scenarioStat = policiesStatsPerScenario.containsKey(scenario.getScenarioName()) ? policiesStatsPerScenario.get(scenario.getScenarioName()) : new HashMap<String,SimulationStatistics>();
		SimulationStatistics stat = scenarioStat.containsKey(policyName) ? scenarioStat.get(policyName) : new SimulationStatistics();
		int nbReceived = nbReceivedByPolicy.containsKey(policyName) ? nbReceivedByPolicy.get(policyName)+1 : 1 ;

		stat.aggregate(name, newStats);
		stat.getCriterion(name).setSimulationParameters(newStats.getCriterion(name).getSimulationParameters());
		Object value = stat.getCriterion(name).getAggregatedValue();
		if(value instanceof Number)
			data.addValue((Number)value, policyName, scenario.getScenarioName());


		curves.put(policyName, series);
		policiesStatsPerScenario.put(scenario.getScenarioName(),scenarioStat );
		scenarioStat.put(policyName, stat);
		nbReceivedByPolicy.put(policyName, nbReceived);
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the data
	 */
	public DefaultCategoryDataset getData() {
		return data;
	}


	
	@Override
	public String toString() {
		String str = "Metric "+name+" : \n";
		for (Entry<String, Map<String, SimulationStatistics>> scenario : policiesStatsPerScenario.entrySet()) {
			str+= "\t"+scenario.getKey()+"[";
			for (Entry<String, SimulationStatistics> policy :scenario.getValue().entrySet()) {
				str+=" "+policy.getKey()+"="+policy.getValue().getAggregatedValue(name)+",";
			}
			str+="]\n";
		}
		
		return str;
	}


	public void saveResults(String path) throws IOException {
		Writer writer = new OutputStreamWriter(new FileOutputStream(path+"/results_"+name+".data"));
		
		List<Entry<String, Map<String, SimulationStatistics>>> sceanarios = new ArrayList<Map.Entry<String,Map<String,SimulationStatistics>>>(policiesStatsPerScenario.entrySet());
		Collections.sort(sceanarios, new Comparator<Entry<String, Map<String, SimulationStatistics>>>() {
			@Override	public int compare(	Entry<String, Map<String, SimulationStatistics>> s1, Entry<String, Map<String, SimulationStatistics>> s2) {
				int e1 = Integer.parseInt(s1.getKey().split("_")[0]);
				int e2 = Integer.parseInt(s2.getKey().split("_")[0]);
				return e1-e2;
			}
		});
		
		for (Entry<String, Map<String, SimulationStatistics>> scenario : sceanarios) {
			writer.write("\""+scenario.getKey()+"\"\t");
			for (Entry<String, SimulationStatistics> policy :scenario.getValue().entrySet()) {
				writer.write("\""+policy.getKey()+"\"\t");
			}
			writer.write("\n");
			break;
		}
		
		for (Entry<String, Map<String, SimulationStatistics>> scenario : sceanarios) {
			writer.write("\""+scenario.getKey()+"\"\t");
			for (Entry<String, SimulationStatistics> policy :scenario.getValue().entrySet()) {
				writer.write(policy.getValue().getAggregatedValue(name)+"\t");
			}
			writer.write("\n");
		}
		writer.close();
	}
	
	
	


}
