package yartiss.service.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

import yartiss.engine.simulation.energy.profiles.consumption.ConsumptionProfileFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;

@SuppressWarnings("serial")
public class TasksTableModel extends AbstractTableModel {

	private static String Type = "Type", r0="r0", Ci="Ci", Ei="Ei", Ti="Ti", Di="Di", Pi="Pi", ConsType="Cons Type";
	private List<Object []> tasks = new LinkedList<Object[]>();
	private String [] columnsNames = new String []{Type, r0, Ci, Ei , Ti, Di, Pi, ConsType};
	private Class<?> [] classes = new Class<?> []{String.class, Integer.class,Integer.class, Integer.class , Integer.class, Integer.class, Integer.class, String.class};
	private boolean [] isVisible   = new boolean [columnsNames.length];
	private Map<String, Integer> mapping = mapHeadersToIndex();
	
	private int prio = 1;
	
	public TasksTableModel(){
		buildIsVisible();
	}
	
	@Override
	public int getRowCount() {
		return tasks.size();
	}

	@Override
	public int getColumnCount() {
		int count = 0;
		for (boolean isvisible : isVisible) {
			count += (isvisible ? 1 : 0);
		}
		return count;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Object [] task = tasks.get(rowIndex);
		
		int index = getMapedIndex(columnIndex);
		if(index >= 0 && index < columnsNames.length)
			return task[index];
		return "-";
	}
	
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		Object [] task = tasks.get(rowIndex);
		Object oldValue = task[columnIndex];
		
		int index = getMapedIndex(columnIndex);
		if(index < 0 || index >= columnsNames.length)
			return;
		
		try {
			task[index] = value;
			getTask(rowIndex);
			//SchedulableFactory.newInstance(task[0].toString(),task[1],task[2],task[3],task[4],task[5],task[6],task[7]);
		}
		catch (Exception e) {
			try {
				task[index] = oldValue;
				getTask(rowIndex);
			} catch (Exception e2) {
				task[index] = 0;
			}
		}
		fireTableChanged(new TableModelEvent(this));
	}
	
	@Override
	public boolean isCellEditable(int row, int col) {
		return true;
	}
	
	public Object [] getRow(int rowIndex){
		return tasks.get(rowIndex);
	}
	
	public void addRow(ITask task){
		addRow(task.getType(), task.getFirstRelease(), task.getWcet(), task.getWcee(), task.getPeriod(), task.getDeadline(), task.getPriority(), task.getEnergyConsumptionProfile().getName());
	}
	
	public ITask getTask(int rowIndex){
		Object [] row = getRow(rowIndex);
		Map<String, Object> values = new HashMap<String, Object>();
		
		values.put("type", row[0].toString());
		values.put("firstrelease", row[1]);
		values.put("wcet", row[2]);
		values.put("wcee", row[3]);
		values.put("period", row[4]);
		values.put("deadline", row[5]);
		values.put("priority", row[6]);
		values.put("constype", row[7]);
		
		if(!isVisible[mapping.get(Ei)]){
			values.put("wcee", 0);
			values.put("constype","default");
		}
		
		ITask newInstance = SchedulableFactory.newInstance(values);
		newInstance.setId(rowIndex+1);
		return newInstance;
	}
	
	public void addRow(Object ... row){
		Object [] task =Arrays.copyOf(row, 8);
		if(task[7]==null)task[7] = getConsumptionFrofiles()[0];
		tasks.add(task);
		fireTableDataChanged();
	}
	
	
	public void addRow(){
		addRow(getSchedulablesTypes()[0],0,1,1,1,1,prio++,getConsumptionFrofiles()[0]);
	}
	
	public void addAll(ITask [] tasksToAdd){
		for (ITask task : tasksToAdd) {
			tasks.add(new Object[]{task.getType(), task.getFirstRelease(), task.getWcet(), task.getWcee(), task.getPeriod(), task.getDeadline(), task.getPriority(), task.getEnergyConsumptionProfile().getName()});
		}
		fireTableDataChanged();
	}
	
	public void removeRow(int rowIndex){
		tasks.remove(rowIndex);
		fireTableDataChanged();
	}
	
	public List<Object[]> getAllItems(){
		return new ArrayList<Object[]>(tasks);
	}
	
	public List<ITask> getAllTasks(){
		List<ITask> allTasks = new ArrayList<ITask>();
		for (int i = 0; i < tasks.size(); i++) {
			allTasks.add( getTask(i) );
		}
		return allTasks;
	}
	
	public String [] getSchedulablesTypes(){
		return SchedulableFactory.availableTypes;
	}
	
	public String [] getConsumptionFrofiles(){
		return ConsumptionProfileFactory.getAvailableProfiles();
	}
	
	public void clear(){
		prio = 1;
		tasks.clear();
		fireTableDataChanged();
	}
	
	@Override
	public String getColumnName(int columnIndex){
		int index = getMapedIndex(columnIndex);
		if(index >= 0 && index < columnsNames.length)
			return columnsNames[index];
		return "-";
	}
	
	@Override
	public Class<?> getColumnClass(int column) {
		int index = getMapedIndex(column);
		if(index >= 0 && index < columnsNames.length)
			return classes[index];
		return String.class;
	}

	public void setEnergyCostVisivble(boolean isvisible) {
		isVisible[mapping.get(Ei)]= isvisible;
		if(!isvisible)
			isVisible[mapping.get(ConsType)]= isvisible;
		fireTableStructureChanged();
	}
	
	public void setConsProfileVisible(boolean isvisible){
		isVisible[mapping.get(ConsType)]= isvisible;
		fireTableStructureChanged();
	}
	
	private Map<String, Integer> mapHeadersToIndex(){
		Map<String, Integer> map = new HashMap<String, Integer>();
		for (int i = 0; i < columnsNames.length; i++) {
			map.put( columnsNames[i], Integer.valueOf(i));
		}
		return map;
	}
	
	private void buildIsVisible() {
		for (int i = 0; i < isVisible.length; i++) {
			isVisible[i] = true;
		}
	}
	
	private int getMapedIndex(int columnIndex){
		int index = -1;
		for (int i = 0; i < columnsNames.length; i++) {
			if(isVisible[i]){
				index++;
				if(columnIndex==index)
					return i;
			}
		}
		return index;
	}

}
