package yartiss.service.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.table.TableModel;

import yartiss.engine.simulation.IEnergyProfile;
import yartiss.engine.simulation.ISchedulingPolicy;
import yartiss.engine.simulation.Simulation;
import yartiss.engine.simulation.SimulationParameters;
import yartiss.engine.simulation.energy.profiles.EnergyProfileFactory;
import yartiss.engine.simulation.events.DeadlineMissedEvent;
import yartiss.engine.simulation.events.EnergyFailureEvent;
import yartiss.engine.simulation.events.EventVisitorAdapter;
import yartiss.engine.simulation.io.many.ITasksetsReader;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.scheduling.policies.SchedulingPolicyFactory;
import yartiss.engine.simulation.statistics.StatisticsManager;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.ITaskSet;
import yartiss.framework.concurrency.AbstractProducer;
import yartiss.framework.concurrency.Consumer;
import yartiss.framework.concurrency.IElementConsumedListener;
import yartiss.framework.concurrency.MultiThreadProducer;
import yartiss.framework.concurrency.QueueEntry;
import yartiss.framework.view.AbstractView;
import yartiss.framework.view.IView;
import yartiss.framework.viewmodel.AbstractViewModel;
import yartiss.service.model.IChartModel;
import yartiss.service.model.Metric;
import yartiss.service.model.MetricsChartModel;
import yartiss.service.model.PoliciesTableModel;
import yartiss.service.model.Scenario;
import yartiss.service.model.ScenariosChartModel;
import yartiss.service.model.ScenariosTableModel;
import yartiss.service.model.TasksSetsTableModel;

public class BenchmarkViewModel extends AbstractViewModel{

	private AbstractView view;
	private int duration = 2550;
	private boolean isContentChanged;
	private boolean isEnergyAware = true;
	private boolean isHardRealtime = true;
	private IChartModel<Scenario> scenariosModel = new ScenariosChartModel();
	private IChartModel<Metric> metricsModel 	 = new MetricsChartModel();
	private final PoliciesTableModel policiesModel = new PoliciesTableModel();
	private final ScenariosTableModel scenariosTableModel = new ScenariosTableModel();

	private Consumer<Simulation> consumer;
	private AbstractProducer<Simulation> producer;
	private boolean debug = false;
	private String systemsFile = "C:\\Users\\Younes\\YounesDev\\YARTISS\\yartiss-gui\\file.txt";//"1000_systems_pr_10";
	private int nbProc = 1;
	private final BoundedRangeModel globalProgressModel = new DefaultBoundedRangeModel(0, 0, 0, 100000000);


	public BenchmarkViewModel(){
		policiesModel.addPolicy("PFP_ALAP_DM", "1");
		policiesModel.addPolicy("PFP_ST_DM", "1");
		policiesModel.addPolicy("PFP_ASAP_DM", "1");

		scenariosTableModel.addScenario(15, 50, 0, 0, true, "constant"); 
		scenariosTableModel.addScenario(15, 100, 0, 0, true, "constant");
		scenariosTableModel.addScenario(15, 200, 0, 0, true, "constant");
		scenariosTableModel.addScenario(15, 300, 0, 0, true, "constant");
		scenariosTableModel.addScenario(15, 400, 0, 0, true, "constant");
		scenariosTableModel.addScenario(15, 600, 0, 0, true, "constant");

	}

	@Override
	public IView getView() {
		return view;
	}

	public void setView(AbstractView view) {
		unregisterViewListeners();
		this.view = view;
		view.setDataContext(this);
		registerViewListeners();		
	}



	private PropertyChangeListener listener = new PropertyChangeListener() {
		@Override	public void propertyChange(PropertyChangeEvent evt) {
			Object value = view.getProperty(evt.getPropertyName());
			setProperty(evt.getPropertyName(), value);
			setIsContentChanged(true);
		}
	};
	private void unregisterViewListeners(){
		if(view != null)
			view.unregisterPropertyChangeListener(listener);
	}

	private void registerViewListeners(){
		if(view != null)
			view.registerPropertyChangeListerner(listener);
	}

	/**
	 * @return the scenariosModel
	 */
	public IChartModel<Scenario> getScenariosModel() {
		return scenariosModel;
	}

	/**
	 * @param scenariosModel the scenariosModel to set
	 */
	public void setScenariosModel(IChartModel<Scenario> scenariosModel) {
		this.scenariosModel = scenariosModel;
		firePropertyChanged("ScenariosModel", scenariosModel, scenariosModel);
	}

	/**
	 * @return the metricsModel
	 */
	public IChartModel<Metric> getMetricsModel() {
		return metricsModel;
	}

	/**
	 * @return the policiesModel
	 */
	public TableModel getPoliciesModel() {
		return policiesModel;
	}


	/**
	 * @return the globalProgressModel
	 */
	public BoundedRangeModel getGlobalProgressModel() {
		return globalProgressModel;
	}

	/**
	 * @return the scenariosTableModel
	 */
	public ScenariosTableModel getScenariosTableModel() {
		return scenariosTableModel;
	}

	/**
	 * @return the isContentChanged
	 */
	public boolean getIsContentChanged() {
		return isContentChanged;
	}

	public void setIsContentChanged(boolean isContentChanged) {
		this.isContentChanged = isContentChanged;
		firePropertyChanged("IsContentChanged", isContentChanged, isContentChanged);
	}

	/**
	 * @return the debug
	 */
	public boolean getDebug() {
		return debug;
	}

	/**
	 * @param debug the debug to set
	 */
	public void setDebug(boolean debug) {
		this.debug = debug;
		firePropertyChanged("Debug", debug, debug);
	}

	/**
	 * @return the duration
	 */
	public int getDuration() {
		return duration;
	}

	/**
	 * @param duration the duration to set
	 */
	public void setDuration(int duration) {
		this.duration = duration;
		firePropertyChanged("Duration", duration, duration);
	}

	/**
	 * @return the nbProc
	 */
	public int getNbProc() {
		return nbProc;
	}

	/**
	 * @param nbProc the nbProc to set
	 */
	public void setNbProc(int nbProc) {
		this.nbProc = nbProc;
		firePropertyChanged("NbProc", nbProc, nbProc);
	}
	
	
	/**
	 * @return the isEnergyAware
	 */
	public boolean getIsEnergyAware() {
		return isEnergyAware;
	}

	/**
	 * @param isEnergyAware the isEnergyAware to set
	 */
	public void setIsEnergyAware(boolean isEnergyAware) {
		this.isEnergyAware = isEnergyAware;
		firePropertyChanged("IsEnergyAware", isEnergyAware, isEnergyAware);
	}

	
	public boolean isHardRealtime() {
		return isHardRealtime;
	}

	public void setHardRealtime(boolean isHardRealtime) {
		this.isHardRealtime = isHardRealtime;
		firePropertyChanged("IsHardRealtime", isHardRealtime, isHardRealtime);
	}
	

	/**
	 * @return the systemsFile
	 */
	public String getSystemsFile() {
		return systemsFile;
	}

	/**
	 * @param systemsFile the systemsFile to set
	 */
	public void setSystemsFile(String systemsFile) {
		this.systemsFile = systemsFile;
		firePropertyChanged("SystemsFile", systemsFile, systemsFile);
	}

	/**
	 * @return the startSimulationCommand
	 */
	public ActionListener getStartSimulationCommand() {
		return startSimulationCommand;
	}

	private void updateDebugResults(Simulation sim, Scenario senario){
		TasksSetsTableModel model = senario.getSetsModel();
		EventsVisitor visitor = new EventsVisitor();
		sim.getStatistics().getLastEvent().accept(visitor);
		model.updateEntry(sim.getSimulationParameters().getTaskSet().getId(), sim.getSimulationParameters().getSchedulingPolicy().getPolicyName(),sim.getSimulationParameters().getTaskSet(),senario.getProfile(),duration,  visitor.result);
	}

	class EventsVisitor extends EventVisitorAdapter{
		public yartiss.service.model.TasksSetsTableModel.Result result = yartiss.service.model.TasksSetsTableModel.Result.SUCCESS;
		@Override
		public void visit(EnergyFailureEvent e) {
			result = yartiss.service.model.TasksSetsTableModel.Result.ENERGY_FAILURE;
		}

		@Override
		public void visit(DeadlineMissedEvent e) {
			result = yartiss.service.model.TasksSetsTableModel.Result.DEADLINE_MISSED;
		}
	}
	private final ActionListener startSimulationCommand = new ActionListener() {
		private long startDate;

		@Override	public void actionPerformed(ActionEvent e) {
			System.out.println("start command");
			setIsContentChanged(false);
			scenariosModel.clear();
			for (Scenario  scenario : scenariosTableModel.getAll(duration)) {
				scenariosModel.add(scenario, scenario.getScenarioName() );
			}

			metricsModel.clear();
			for (String metric : StatisticsManager.getActivatedCriteria()) {
				metricsModel.add(new Metric(metric), metric);
			}

			globalProgressModel.setValue(0);

			startDate = System.currentTimeMillis();
			new Thread(new Runnable() {
				int count;
				int total;
				boolean finish = false;
				public void run() {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							count = 0;
							finish = false;
							SwingWorker<Void, String> task = new SwingWorker<Void, String>() {
								private final BlockingQueue<QueueEntry<Simulation>> queue = new LinkedBlockingQueue<QueueEntry<Simulation>>(3000);
								@Override	protected Void doInBackground() throws Exception {

									producer = new MultiThreadProducer<Simulation>(queue);
									consumer = new Consumer<Simulation>(queue, new IElementConsumedListener<Simulation>() {
										@Override
										public void consume(Simulation sim) {  
											count ++;

											if(finish && count==total){
												producer.produce(null);
											}

											String scenarioName = sim.getSimulationParameters().getEnergyProfile().getMaxEnergyLevel()+"_"+sim.getSimulationParameters().getEnergyProfile().getPowerRate();
											Scenario scenario = scenariosModel.getElementByName(scenarioName);
											scenario.updatePolicyStats(sim.getSimulationParameters(), sim.getStatistics());
											if(debug)
												updateDebugResults(sim, scenario);
											for (String metric : StatisticsManager.getActivatedCriteria()) {
												Metric m = metricsModel.getElementByName(metric);
												m.updatePolicyStats(scenario, sim.getSimulationParameters().getSchedulingPolicy().getPolicyName(), sim.getStatistics());
											}
											globalProgressModel.setValue(globalProgressModel.getValue()+1);
											//System.out.println((((double)globalProgressModel.getValue())/globalProgressModel.getMaximum())*100);
										}
									});
									consumer.addFinishListener(new IElementConsumedListener<Simulation>() {
										@Override	public void consume(Simulation element) {
											try {
												saveResaults();
											} catch (IOException e) {
												e.printStackTrace();
											}
											System.out.println("finish : count="+count);
											long endtDate = System.currentTimeMillis();
											System.out.println("total simulation duration : "+ (endtDate-startDate)/1000);
										}
									});

									consumer.start();

									for (Scenario s : scenariosTableModel.getAll(duration)) {
										IEnergyProfile ep  = s.getProfile();
										Scenario scenario = scenariosModel.getElementByName(ep.getMaxEnergyLevel()+ "_" + ep.getPowerRate());
										System.out.println("start ... producing");
										for (ISchedulingPolicy policy : policiesModel.getAll(duration)) {
											System.out.println("systems file ="+ systemsFile);
											ITasksetsReader reader = null;
											if(systemsFile.contains("file:") || systemsFile.contains("jar:")){	 
												URL url = new URL(systemsFile);
												reader = TasksetsIOFactory.newTasksetsReaderInstance("txt", url.openStream());
											}
											else {
												
												if(systemsFile.endsWith(".xml"))
													reader = TasksetsIOFactory.newTasksetsReaderInstance("xml",systemsFile);
												else
													reader = TasksetsIOFactory.newTasksetsReaderInstance("txt",systemsFile);
											}

											int nbEntries = 0;
											System.out.println("-----------" + policy.getPolicyName()+ " :  started");
											ITask[] tasks = null;
											while((tasks = reader.nextTaskset()) != null){

												nbEntries ++;
												final List<Integer> paramss = policy.getParameters()==null ? null : new LinkedList<Integer>(policy.getParameters());
												final ISchedulingPolicy policy2 = SchedulingPolicyFactory.getInstance(policy.getPolicyName());
												final ITask[] tasks2 = tasks;
												final int id = nbEntries;
												policy2.setParameters(paramss);
												final IEnergyProfile fep = EnergyProfileFactory.getInstance("defaultEnergyProfile", (int)ep.getPowerRate(), (int)ep.getMaxEnergyLevel(), (int)ep.getMinEnergyLevel(), (int)ep.getStartEnergyLevel());
												producer.produce(new Callable<Simulation>() {
													@Override
													public Simulation call() throws Exception {
														Simulation sim = new Simulation(buildSimulationParameters(fep, policy2, tasks2, id, getNbProc()));
//														if(sim.getSimulationParameters().getTaskSet().getId()>122)
//															SimulationParametersIOFactory.newSimulationParametersWriterInstance("xml").write(sim.getSimulationParameters(), "C:/Users/Younes/Desktop/sysem-"+sim.getSimulationParameters().getTaskSet().getId()+".xml");
														
														sim.simulate();
														//System.out.println("id="+Thread.currentThread().getId()+"    name="+Thread.currentThread().getName()+" system="+sim.getSimulationParameters().getTaskSet().getId());
														return sim;
													}
												});
												scenario.setSystemsCount(nbEntries) ;
											}
											total = nbEntries*policiesModel.getRowCount()*scenariosTableModel.getRowCount();
											reader.close();
											for (Scenario ci : scenariosTableModel.getAll(duration)) {
												Scenario nario = scenariosModel.getElementByName(ci.getScenarioName());
												nario.setSystemsCount(nbEntries) ;
											}
											globalProgressModel.setMaximum(nbEntries*policiesModel.getRowCount()*scenariosTableModel.getRowCount());
										}
									}
									finish = true;
									System.out.println("finish producing : "+total);
									return null;
								}
							};
							task.execute();
						}
					});
				}
			}).start();
			
		}
	};


	private void saveResaults() throws IOException{
		Writer writer;
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd--HH-mm-ss");
		String resultsArchive = "results_"+dateFormat.format(new Date());
		String path = System.getProperty("user.home") + "/"+resultsArchive;
		path = path.replace("\\", "/");
		System.out.println(path);
		File file = new File(path);
		if(!file.exists())
			file.mkdirs();

		for (File f : file.listFiles()) {
			f.delete();
		}

		writer = new OutputStreamWriter(new FileOutputStream(path+"/results_per_scenario.gnuplot"));
		writer.write("set boxwidth 0.9 absolute\n");
		writer.write("set style fill   solid 1.00 border lt -1\n");
		writer.write("set key inside right top vertical Right noreverse noenhanced autotitles nobox\n");
		writer.write("set style histogram clustered gap 1 title  offset character 0, 0, 0\n");
		writer.write("set datafile missing '-'\n");
		writer.write("set style data histograms\n");
		writer.write("set xtics border in scale 0,0 nomirror rotate by -45  offset character 0, 0, 0 autojustify\n");
		writer.write("set xtics  norangelimit font \",8\"\n");
		writer.write("set xtics   ()\n");
		writer.write("set style histogram gap 5\n");
		writer.write("set grid y\n");

		int termId = 0;
		for (Scenario scenario : scenariosModel.getAll()) {
			scenario.saveResults(path);

			writer.write("set terminal windows "+termId+"\n");
			termId++;
			writer.write("set title \"Scenario :"+scenario.getScenarioName()+"\" \n");
			String plot = "plot\t'results_"+scenario.getScenarioName()+".data' ";
			int i=2;
			for (String criterion : StatisticsManager.getActivatedCriteria()) {
				if(StatisticsManager.getInstance(criterion).getValue() instanceof Number){
					if(i>2)plot += "'' ";
					plot += "using "+i+":xtic(1) ti col";
					i++;
					if(i-2 < StatisticsManager.getActivatedCriteria().length-1)plot += ", ";
				}
			}
			writer.write(plot+"\n\n");
		}
		writer.close();

		writer = new OutputStreamWriter(new FileOutputStream(path+"/results_per_metric.gnuplot"));

		writer.write("set datafile missing \"-\"\n");
		writer.write("set xtics nomirror rotate by -45 font \",8\"\n");
		writer.write("set key noenhanced\n");
		writer.write("set style data linespoints\n");

		for (String metric : StatisticsManager.getActivatedCriteria()) {
			if(StatisticsManager.getInstance(metric).getValue() instanceof Number){
				Metric m = metricsModel.getElementByName(metric);
				m.saveResults(path);

				writer.write("set terminal windows "+termId+"\n");
				termId++;
				writer.write("set title \"Metric :"+m.getName()+"\" \n");
				String plot = "plot\t'results_"+m.getName()+".data' ";
				int i=2;
				for (Scenario scenario : scenariosModel.getAll()) {
					if(i>2)plot += "'' ";
					plot += "using "+i+":xtic(1) ti col";
					i++;
					if(i-2 < scenariosModel.getAll().size())plot += ", ";
				}
				writer.write(plot+"\n\n");
			}
		}
		writer.close();

		writer = new OutputStreamWriter(new FileOutputStream(path+"/all_results.gnuplot"));
		writer.write("load 'results_per_scenario.gnuplot'\n");
		writer.write("load 'results_per_metric.gnuplot'\n");
		for (Scenario scenario : scenariosModel.getAll()) {
			writer.write(" load 'results3D_"+scenario.getScenarioName()+".gnuplot'\n");
		}
		writer.close();
	}


	private SimulationParameters buildSimulationParameters(IEnergyProfile energyProfile, ISchedulingPolicy policy, ITask[] tasks, int id, int nbProc) {
		ITaskSet taskset = policy.createTaskSet();
		taskset.setId(id);
		for (ITask task : tasks) {
			taskset.addTask(task.cloneTask());
		}
		return new SimulationParameters(policy, duration ,	energyProfile, taskset,nbProc, isEnergyAware, isHardRealtime);
	}


	public static void zipDir(String dir2zip, ZipOutputStream zos) throws IOException 
	{ 
		//create a new File object based on the directory we        have to zip File    
		File zipDir = new File(dir2zip); 
		//get a listing of the directory content 
		String[] dirList = zipDir.list(); 
		byte[] readBuffer = new byte[2156]; 
		int bytesIn = 0; 
		//loop through dirList, and zip the files 
		for(int i=0; i<dirList.length; i++) 
		{ 
			File f = new File(zipDir, dirList[i]); 
			if(f.isDirectory()) 
			{ 
				//if the File object is a directory, call this 
				//function again to add its content recursively 
				String filePath = f.getPath(); 
				zipDir(filePath, zos); 
				//loop again 
				continue; 
			} 
			//if we reached here, the File object f was not a directory 
			//create a FileInputStream on top of f 
			FileInputStream fis = new FileInputStream(f); 
			//create a new zip entry 
			ZipEntry anEntry = new ZipEntry(f.getPath()); 
			//place the zip entry in the ZipOutputStream object 
			zos.putNextEntry(anEntry); 
			//now write the content of the file to the ZipOutputStream 
			while((bytesIn = fis.read(readBuffer)) != -1) 
			{ 
				zos.write(readBuffer, 0, bytesIn); 
			} 
			//close the Stream 
			fis.close(); 
		} 
	}


}
