package yartiss.service.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import java.util.TreeSet;

import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;

import yartiss.engine.simulation.energy.profiles.consumption.ConsumptionProfileFactory;
import yartiss.engine.simulation.io.many.ITasksetsWriter;
import yartiss.engine.simulation.io.many.TasksetsIOFactory;
import yartiss.engine.simulation.system.ITask;
import yartiss.engine.simulation.system.SystemMetrics;
import yartiss.engine.simulation.system.schedulable.SchedulableFactory;
import yartiss.engine.simulation.system.taskset.generation.ControledHyperPerdiodUUniFastDiscardGen;
import yartiss.framework.view.AbstractView;
import yartiss.framework.view.IView;
import yartiss.framework.viewmodel.AbstractViewModel;

public class TaskSetsGenViewModel extends AbstractViewModel{

	private boolean isContentChanged = true;
	private boolean isGlobalConsumption = true;
	private boolean variateU=true;
	private int nbProc = 1;
	private int Pr=15;
	private int count=100;
	private int Emax=200;
	private double utilization=0.7, energyUtilization=0.7;
	private int nbTasks=4;
	private String selectedConsumptionProfile=ConsumptionProfileFactory.getAvailableProfiles()[0];
	private String selectedTaskType = getAvailibaleTaskTypes()[0];
	private AbstractView view;
	private final BoundedRangeModel globalProgressModel = new DefaultBoundedRangeModel(0, 0, 0, count);
	private String systemsFile = "file.txt";

	@Override
	public IView getView() {
		return view;
	}

	public void setView(AbstractView view) {
		unregisterViewListeners();
		this.view = view;
		view.setDataContext(this);
		registerViewListeners();		
	}

	private PropertyChangeListener listener = new PropertyChangeListener() {
		@Override	public void propertyChange(PropertyChangeEvent evt) {
			Object value = view.getProperty(evt.getPropertyName());
			setProperty(evt.getPropertyName(), value);
			setIsContentChanged(true);
		}
	};

	private void unregisterViewListeners(){
		if(view != null)
			view.unregisterPropertyChangeListener(listener);
	}

	private void registerViewListeners(){
		if(view != null)
			view.registerPropertyChangeListerner(listener);
	}

	/**
	 * @return the isContentChanged
	 */
	public boolean getIsContentChanged() {
		return isContentChanged;
	}

	public void setIsContentChanged(boolean isContentChanged) {
		this.isContentChanged = isContentChanged;
		firePropertyChanged("IsContentChanged", isContentChanged, isContentChanged);
	}


	public String [] getAvailibaleConsumptionProfiles() {
		return ConsumptionProfileFactory.getAvailableProfiles();
	}

	public String [] getAvailibaleTaskTypes() {
		return SchedulableFactory.availableTypes;
	}

	/**
	 * @return the isGlobalConsumption
	 */
	public boolean getIsGlobalConsumption() {
		return isGlobalConsumption;
	}

	/**
	 * @param isGlobalConsumption the isGlobalConsumption to set
	 */
	public void setIsGlobalConsumption(boolean isGlobalConsumption) {
		this.isGlobalConsumption = isGlobalConsumption;
		firePropertyChanged("IsGlobalConsumption", isGlobalConsumption, isGlobalConsumption);
	}


	/**
	 * @return the selectedPolicyName
	 */
	public String getSelectedConsumptionProfile() {
		return selectedConsumptionProfile;
	}

	/**
	 * @param selectedPolicyName the selectedPolicyName to set
	 */
	public void setSelectedConsumptionProfile(String selectedConsumptionProfile) {
		this.selectedConsumptionProfile = selectedConsumptionProfile;
		firePropertyChanged("SelectedConsumptionProfile", selectedConsumptionProfile, selectedConsumptionProfile);
	}

	/**
	 * @return the selectedPolicyName
	 */
	public String getSelectedTaskType() {
		return selectedTaskType;
	}

	/**
	 * @param selectedPolicyName the selectedPolicyName to set
	 */
	public void setSelectedTaskType(String selectedTaskType) {
		this.selectedTaskType = selectedTaskType;
		firePropertyChanged("SelectedTaskType", selectedTaskType, selectedTaskType);
	}

	/**
	 * @return the nbProc
	 */
	public int getNbProc() {
		return nbProc;

	}

	/**
	 * @param nbProc the nbProc to set
	 */
	public void setNbProc(int nbProc) {
		this.nbProc = nbProc;
		firePropertyChanged("NbProc", nbProc, nbProc);
	}

	/**
	 * @return the emax
	 */
	public int getEmax() {
		return Emax;
	}

	/**
	 * @param emax the emax to set
	 */
	public void setEmax(int emax) {
		Emax = emax;
		firePropertyChanged("Emax", emax, emax);
	}

	/**
	 * @return the pr
	 */
	public int getPr() {
		return Pr;
	}

	/**
	 * @param pr the pr to set
	 */
	public void setPr(int pr) {
		Pr = pr;
		firePropertyChanged("Pr", pr, pr);
	}

	/**
	 * @return the count
	 */
	public int getCount() {
		return count;
	}

	/**
	 * @param count the count to set
	 */
	public void setCount(int count) {
		this.count = count;
		globalProgressModel.setMaximum(count);
		firePropertyChanged("Count", count, count);
	}

	/**
	 * @return the nbTasks
	 */
	public int getNbTasks() {
		return nbTasks;
	}

	/**
	 * @param nbTasks the nbTasks to set
	 */
	public void setNbTasks(int nbTasks) {
		this.nbTasks = nbTasks;
		firePropertyChanged("NbTasks", nbTasks, nbTasks);
	}

	/**
	 * @return the utilization
	 */
	public double getUtilization() {
		return utilization;
	}

	/**
	 * @param utilization the utilization to set
	 */
	public void setUtilization(double utilization) {
		this.utilization = utilization;
		firePropertyChanged("Utilization", utilization, utilization);
	}

	/**
	 * @return the energyUtilization
	 */
	public double getEnergyUtilization() {
		return energyUtilization;
	}

	/**
	 * @param energyUtilization the energyUtilization to set
	 */
	public void setEnergyUtilization(double energyUtilization) {
		this.energyUtilization = energyUtilization;
		firePropertyChanged("EnergyUtilization", energyUtilization, energyUtilization);
	}

	/**
	 * @return the variateU
	 */
	public boolean getVariateU() {
		return variateU;
	}

	/**
	 * @param variateU the variateU to set
	 */
	public void setVariateU(boolean v) {
		this.variateU = v;
		firePropertyChanged("VariateU", variateU, variateU);
		System.out.println("variateU="+variateU);
	}


	/**
	 * @return the systemsFile
	 */
	public String getSystemsFile() {
		return systemsFile;
	}

	/**
	 * @param systemsFile the systemsFile to set
	 */
	public void setSystemsFile(String systemsFile) {
		this.systemsFile = systemsFile;
		firePropertyChanged("SystemsFile", systemsFile, systemsFile);
	}


	/**
	 * @return the globalProgressModel
	 */
	public BoundedRangeModel getGlobalProgressModel() {
		return globalProgressModel;
	}

	/**
	 * @return the generateCommand
	 */
	public ActionListener getGenerateCommand() {
		return generateCommand;
	}

	private final ActionListener generateCommand = new ActionListener() {
		@Override public void actionPerformed(ActionEvent e) {
			new Thread(new Runnable() {
				public void run() {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							SwingWorker<Void, String> task = new SwingWorker<Void, String>() {
								@Override	protected Void doInBackground() throws Exception {
									globalProgressModel.setValue(0);
									int n=20 ;
									float start = 0.2f;
									int counter = 0;
									float unit = 1.f/n;
									int s = Math.round(start/unit);
									int maxIndex = n;
									int nb =  Math.max(1, count / ((maxIndex+1-s)*(maxIndex+1-s)) );
									if(variateU)
										setCount((maxIndex+1-s)*(maxIndex+1-s));
									System.out.println(s);
									//System.exit(0);

									if(!variateU){
										nb = count;
										s=(int) Math.round(Math.min(energyUtilization,utilization)/unit);
									}
									globalProgressModel.setMaximum(count);

									TreeSet<ITask[]> [][] stats = new TreeSet[n+1][n+1];

									for (int i = 0; i < stats.length; i++)
										for (int j = 0; j < stats.length; j++)
											stats[i][j]=new TreeSet<ITask[]>(new Comparator<ITask[]>() {
												@Override
												public int compare(ITask[] set1, ITask[] set2) {
													if(set1.length != set2.length)return 1;
													for (int i = 0; i < set1.length; i++) {
														if ( 	set1[i].getFirstRelease() != set2[i].getFirstRelease()
																||	set1[i].getWcet() != set2[i].getWcet()
																||	set1[i].getWcee() != set2[i].getWcee()
																||	set1[i].getPeriod() != set2[i].getPeriod()
																||	set1[i].getDeadline() != set2[i].getDeadline()
																//||	set1[i].getPriority() != set2[i].getPriority()
																)
															return 1;
													}
													return 0;
												}
											});

											int maxIterations = 10000;
											int iterationCounter = 0;
											do{
												iterationCounter++;
												for (long i = s; i <= maxIndex; i++) {
													for (long j = s; j <= maxIndex; j++) {
														for (int k = 0; k < nb; k++) {
															double u  =  Math.round((unit*i) * 1000) / 1000.0d;
															double ue =  Math.round((unit*j) * 1000) / 1000.0d;
															//System.out.println("---- u="+u+"	ue="+ue);

															if(!variateU){
																u = utilization;
																ue = energyUtilization;
															}

															int line = (int) Math.round(u / unit);
															int row = (int) Math.round(ue / unit);


															if(stats[line][row].size() < nb){
																ControledHyperPerdiodUUniFastDiscardGen gen = new ControledHyperPerdiodUUniFastDiscardGen(nbTasks,1, u, ue, Pr, Emax, 0, false);
																ITask [] taskset = gen.generate();

																u  = Math.round(SystemMetrics.utilisation(Arrays.asList(taskset)) * 1000) / 1000.0d;
																ue = Math.round(SystemMetrics.energyUtilization(Arrays.asList(taskset), Pr) * 1000) / 1000.0d;
																
																if(!variateU){
																	u = utilization;
																	ue = energyUtilization;
																}

																line = (int) Math.round(u / unit);
																row = (int) Math.round(ue / unit);

																if(line >= s && row >= s &&  line <= maxIndex    &&    row <= maxIndex    &&   u<=1 && ue<=1   &&   stats[line][row].size() < nb){

																	int oldSize = stats[line][row].size();
																	stats[line][row].add(taskset);

																	if(oldSize < stats[line][row].size()){
																		counter++;
																		globalProgressModel.setValue(counter);
																	}
																	else {
																		//System.err.println("doublon supprimé");
																	}
																}
															}
														}
													}
												}

												//if(counter > oldCounter)
												//	flush(stats, destFile, pr);

												if(iterationCounter % maxIterations==0)
													nbTasks = nbTasks==2 ? nbTasks : nbTasks-1;

												//System.out.println("("+line+" , "+row+")"+"    u="+u+"	ue="+ue+"  hp="+SystemMetrics.hyperPeriod(Arrays.asList(taskset)));
												System.out.println("---------------------------------------"+counter+"/"+count);

												//showTable(stats, unit);


											}while(counter < count);

											flush(stats, systemsFile , Pr);
											return null;
								}
							};
							task.execute();
						}
					});
				}
			}).start();
		}
	};

	private static void flush(TreeSet<ITask[]> [][] stats, String destFile, long pr) throws Exception{

		String type = "txt";
		if(destFile.endsWith(".xml"))
			type = "xml";
		final ITasksetsWriter writer = TasksetsIOFactory.newTasksetsWriterInstance(type, destFile, pr);
		for (int i = 0; i < stats.length; i++){
			for (int j = 0; j < stats.length; j++){
				for (ITask[] taskset : stats[i][j]) {
					writer.write(taskset);
				}
				stats[i][j].clear();
				stats[i][j] = null;
			}
		}
		writer.close();
		System.out.println("writting on :"+destFile);
	}

	private static void showTable(TreeSet<ITask[]> [][] stats, double unit){
		System.out.print("ue/e\t");
		for (int p = 0; p < stats.length; p++) {
			System.out.print(Math.round(p*unit * 1000)/1000.0d  +"\t");
		}
		System.out.println();
		for (int p = 0; p < stats.length; p++) {
			System.out.print(Math.round(p*unit * 1000)/1000.0d +"\t");
			for (int m = 0; m < stats[0].length; m++) {
				System.out.print(stats[p][m].size()+"\t");
			}
			System.out.println();
		}
	}

	public static void main(String[] args) {
		File f = new File("./");
		System.out.println(f.getAbsolutePath());
		TaskSetsGenViewModel gen = new TaskSetsGenViewModel();
		gen.getGenerateCommand().actionPerformed(null);
	}
}
